#!/bin/bash
MY_VERSION="Version : v3.5 release in 2024-12-20"
MY_PGSQL=/home/postgres/pgsql/bin/psql
MY_UNIPD=/usr/local/bin/unipd
MY_PASSWD=/usr/local/bin/univpasswd

DB_USER=postgres
DB_PGPORT=5432
DB_PGHOST=127.0.0.1
DB_CONF=/mnt/syncdata/pgsql/data/postgresql.conf
DB_PGHBA_CONF=/mnt/syncdata/pgsql/data/pg_hba.conf
DB_PASSWD_CONF=/usr/local/svconfig/server/conf/dbpasswd.conf
DB_BACKUP_DIR=/var/autobackup

IP_INFO=127.0.0.1
OLD_PATH=` pwd `
SHELL_DIR=$(dirname $(readlink -f "$0"))
SHELL_NAME=${0##*/}
SHELL_PATH="${SHELL_DIR}/${SHELL_NAME}"
PID_FILE="${SHELL_DIR}/GeneralCheck.pid"
LOG_DIR=${SHELL_DIR}
CHECK_FILE=check.log
CHECK_CSV=check.csv
TEMP_FILE=/tmp/GeneralCheck_tmp.txt
AUTO_START_CONFIG=/etc/rc.d/rc.local
IMOS_CFG=/usr/local/svconfig/server/conf/imos.cfg
IMOS_CONFIG_DIR=/usr/local/imosconfig
HA_CORE_DIR=/var/lib/pacemaker/cores/  # HA core dir
HA_PATCH_LOG=/var/log/logpthrecord.log # Patch CHECK_FILE
GATEWAY=` route -n | grep -e "^0.0.0.0" | head -1 | awk -F' ' '{print $2}' `
CentOS_VER=` sed -r 's/.* ([0-9]+)\..*/\1/' /etc/redhat-release `

# flag
CHECK_NUM=1         # sequence in every part of check item
IPCNum_onvif_pg=0   # number of onvif IPC in PG
IPCNum_guobiao_pg=0 # number of guobiao IPC in PG
TIMING_JOB=1        # 1 : timing-job mode off(default)
                    # 0 : timing-job mode on
CHECK_EXPIRED=0     # 0 : check whether script has expired(default)
                    # 1 : do not check
CHECK_CSV_FLAG=0    # when write check.csv in first time, flag change to 1

source /etc/profile >/dev/null 2>&1

check_item_array=(
    "System_Info"     # 1
    "Process_Status"  # 2
    "Disk_Space"      # 3
    "Network_Status"  # 4
    "PG_Status"       # 5
    "HA_Status"       # 6
    "VM_Status"       # 7
    "System_Log"      # 8
    "Tool_Deployment" # 9
)

pg_data_dir_array=(` ls -ld /mnt/syncdata/pgsql/* 2>/dev/null | grep '^d' | awk -F' ' '{print $9}' `)
if [ -z "` /home/postgres/pgsql/bin/psql -V | grep " 11." `" ]
then
    pg_data_wal_dir_array=(` ls -ld /mnt/syncdata/pgsql/*/pg_xlog 2>/dev/null | grep '^d' | awk -F' ' '{print $9}' `)
else
    pg_data_wal_dir_array=(` ls -ld /mnt/syncdata/pgsql/*/pg_wal  2>/dev/null | grep '^d' | awk -F' ' '{print $9}' `)
fi
dir_array=(
    "/dev/null"
    "/tmp"
    "/var"
    "/var/log"
    "/var/log/imoslog/core_dump_manager"
    "/var/lib"
    "/var/lib/pacemaker/cores"
    "/var/crash"
    "/var/autobackup"
    "/mnt/syncdata"
    "/mnt/syncdata/resftp"
    "/mnt/syncdata/resftp/A8"
    ${pg_data_dir_array[@]}
    ${pg_data_wal_dir_array[@]}
)
pg_data_mod_array=(` for i in ${pg_data_dir_array[@]} ; do echo "drwx------,postgres,postgres " ; done `)
pg_data_wal_mod_array=(` for i in ${pg_data_wal_dir_array[@]} ; do echo "drwx------,postgres,postgres " ; done `)
dir_mod_array=(
    "crw-rw-rw-,root,root"            # /dev/null
    "drwxrwxrwt,root,root"            # /tmp
    "drwxr-xr-x,root,root"            # /var
    "drwxr-xr-x,root,root"            # /var/log
    "drwxr-xr-x,root,root"            # /var/log/imoslog/core_dump_manager
    "drwxr-xr-x,root,root"            # /var/lib
    "drwxr-x---,hacluster,haclient"   # /var/lib/pacemaker/cores
    "drwxr-xr-x,root,root"            # /var/crash
    "drwxr-xr-x,root,root"            # /var/autobackup
    "drwxr-xr-x,root,root"            # /mnt/syncdata
    "drwxrwx---,downloadusr,h3cgroup" # /mnt/syncdata/resftp
    "drwxrwxr-x,uploadusr,h3cgroup"   # /mnt/syncdata/resftp/A8
    ${pg_data_mod_array[@]}
    ${pg_data_wal_mod_array[@]}
)

pacemaker_process_array=(
    "cib"
    "stonithd"
    "lrmd"
    "attrd"
    "pengine"
    "crmd"
)

check_log_array=()
check_csv_array=()

my_wait()
{
    [ $1 ] && [ $1 -eq 0 ] && CHECK_NUM=1
    printf '\b\b\b\b(%2d)' ${CHECK_NUM}
    let "CHECK_NUM+=1"
}

is_Rsync()
{
    if   [ ""X != "` grep '^hot_standby = on' ${DB_CONF} `"X ]
    then
        return 2 # rsync
    elif [ -f "/sbin/drbdadm" ]
    then
        return 3 # drbd
    else
        return 1
    fi
}

Get_DBpasswd()
{
    read_passwd_old()
    {
        if [ -f "${DB_PASSWD_CONF}" ]
        then
            . ${DB_PASSWD_CONF}
            ${MY_PASSWD} "2" "${dbpasswd}"
            if [ -f "dbpasswd_temp.conf"  ]
            then
                . ${SHELL_DIR}/dbpasswd_temp.conf
                export PGUSER=postgres
                export PGPASSWORD=${dbpasswd}
                rm -rf dbpasswd_temp.conf 2>/dev/null
            else
                echo -e "\n--[error]-- Get_DBpasswd : read DB passwd error, file dbpasswd_tmp.conf does not exist \n"
                export PGUSER=postgres
                export PGPASSWORD=passwd
            fi
        else
            export PGUSER=postgres
            export PGPASSWORD=passwd
        fi
    }

    if [ -f "${MY_UNIPD}" ]
    then
        if [ -f "${IMOS_CFG}" ]
        then
            dbpasswd=` cat ${IMOS_CFG} | grep DBPassword | awk -F'=' '{print $2}' `
            dbpasswd=` ${MY_UNIPD} 1 ${dbpasswd} `
            if [ ""X == "${dbpasswd}"X ]
            then
                echo -e "\n--[error]-- Get_DBpasswd : read DB passwd error, DBPassword not in imos.cfg \n"
                read_passwd_old
            else
                export PGUSER=postgres
                export PGPASSWORD=${dbpasswd}
            fi
        elif [ -f "/usr/local/svconfig/server/conf/imos_cds.cfg" ]
        then
            dbpasswd=` cat /usr/local/svconfig/server/conf/imos_cds.cfg | grep DBPassword | awk -F'=' '{print $2}' `
            dbpasswd=` ${MY_UNIPD} 1 ${dbpasswd} `
            if [ ""X == "${dbpasswd}"X ]
            then
                echo -e "\n--[error]-- Get_DBpasswd : read DB passwd error, DBPassword not in imos_cds.cfg \n"
                read_passwd_old
            else
                export PGUSER=postgres
                export PGPASSWORD=${dbpasswd}
            fi
        else
            read_passwd_old
        fi
    else
        read_passwd_old
    fi
}

Write_to_CHECK_CSV()
{
    # $1 : echo option para
    # $2 : info to be written
    if [ "${CHECK_CSV_FLAG}" -eq 0 ]
    then
        cat > ${CHECK_CSV} << EOF
Time : ` date `,Ip : ${IP_INFO},${MY_VERSION}
Check Item,Auto-Check Reault,Result Detail
EOF
        CHECK_CSV_FLAG=1
    fi
    cat >> ${CHECK_CSV} << EOF
"$1","$2","$3"
EOF
}

Core_Analysis()
{
    core_file_list=core.list
    core_info_output=core.info
    core_file_with_keyword=(
        "/root/core"
        "/var/lib/pacemaker/cores/core")
    echo > ${core_info_output}
    for core_key in ${core_file_with_keyword[@]}
    do
        analysis_day=10 # check recent 10 days core file
        analysis_num=2  # check newest 2 core file
        find ${core_key}* -type f -mtime -${analysis_day} 2>/dev/null | xargs -I {} file {} 2>/dev/null | grep "/usr/local/bin/" > ${core_file_list} 2>/dev/null

        while read line
        do
            # echo "${analysis_num}"
            [ ${analysis_num} -lt 0 ] && break
            core_file=` echo ${line} | awk -F':' '{print $1}' `
            file_size=` du -sh ${core_file} | awk -F' ' '{print $1}' `
            process_name=` echo ${line} | awk -F'from' '{print $2}' `
            # echo ${process_name}
            # gdb details commands
            #     info all-registers
            #     info args
            #     info locals
            #     info catch
            cat >> ${core_info_output} << TITLE
***************************************************************
debug   : ` gdb -v | head -1 `
core    : ${core_file}
size    : ${file_size}
process :${process_name}

TITLE

            # analysis the top 3 of stack with frame,threads,registers,args,locals,catch
            gdb ${process_name} ${core_file} >> ${core_info_output} 2>/dev/null << GDB
thread apply all bt
quit
GDB
            # [ $? -eq 0 ] && let "analysis_num-=1"
            let "analysis_num-=1"
            echo -e "\n***************************************************************" >> ${core_info_output}
        done < ${core_file_list}

    done
    sed -i 's/(gdb) /\n(gdb) \n/'    ${core_info_output}
    # sed -i '/Reading symbols from/d' ${core_info_output}
    # sed -i '/Loaded symbols for/d'   ${core_info_output}
    cat ${core_info_output} >> ${CHECK_FILE}
    /bin/rm -rf ${core_file_list}   2>/dev/null
    /bin/rm -rf ${core_info_output} 2>/dev/null
}

System_Info()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : System Info
EOF


    my_wait 0 # System Basic Info
    cat > ${TEMP_FILE} << EOF

------  checklist : system basic info  ---------
System Ver    : ` cat /etc/redhat-release `  ` getconf LONG_BIT ` bit
System Core   : ` nproc `
Cpu Info      :` grep "model name" /proc/cpuinfo | head -1 | awk -F':' '{print $2}' `
Start-up Time : ` date -d "$(awk -F'.' '{print $1}' /proc/uptime) second ago" +"%Y-%m-%d %H:%M:%S" `
Running Time  :` uptime `
Host Name     : ` hostname `
System Time   : ` date | awk '{printf("%s %02s %s %s %s %s",$1,$3,$2,$6,$4,$5)}' `
Hardware Time : ` hwclock --localtime 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Grub
    if   [ "${CentOS_VER}" -eq 7 ]
    then
        grep "GRUB_CMDLINE_LINUX" /etc/default/grub | grep -q "LANG=" ; temp_rc1=$?
        grep "GRUB_CMDLINE_LINUX" /etc/default/grub | grep -q "acpi_enforce_resources=no" ; temp_rc2=$?
    else
        grep -q "LANG=" /etc/grub.conf ; temp_rc1=$?
        grep -q "acpi_enforce_resources=no" /etc/grub.conf ; temp_rc2=$?
    fi
    
    if [ ${temp_rc1} -eq 1 ] && [ ${temp_rc2} -eq 0 ]
    then # 不存在LANG配置项 但存在acpi_enforce_resources配置项 配置正确
        check_result="[ correct ]"
    else
        check_result="[ wrong ]"
    fi

    cat > ${TEMP_FILE} << EOF

------  checklist : grub  ----------------------
------  RESULT(grub) : ${check_result}  ------
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}


    my_wait # Cpu Speed
    if   [ "${CentOS_VER}" -eq 7 ]
    then
        temp_cpu_speed=`systemctl status cpupower `
        check_status=`  systemctl status cpupower | grep -e " inactive (dead)" `
    else
        temp_cpu_speed=`service cpuspeed status 2>/dev/null `
        check_status=`  service cpuspeed status 2>/dev/null | grep -e " stop" `
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : cpu speed  -----------------
${temp_cpu_speed}
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ -z "${check_status}" ]
    then
        echo "------  RESULT(cpu speed) : [ wrong ]  ------"  >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "cpu speed limit is on"
    else
        echo "------  RESULT(cpu speed) : [ correct ]  ------">> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Memory Hardware Info
    temp_memory_all=`  dmidecode -t 17 2>/dev/null | grep -e "Memory Device" | wc -l `
    temp_memory_used=` dmidecode -t 17 2>/dev/null | grep -e "Size: " | grep -v "No Module" | wc -l `
    cat > ${TEMP_FILE} << EOF

------  checklist : memory hardware info  ------
` dmidecode -t 17 2>/dev/null | grep -e "Memory Device" -e "Size:" -e "Type:" -e $'\t''Speed:' | grep -v "No Module" | grep -v "Unknown" | awk '{if ($0!=line) print;line=$0}' `
Memory Slot(Used/All) : ${temp_memory_used} / ${temp_memory_all}
Maximum Capacity      :` dmidecode 2>/dev/null | grep -e "Maximum Capacity" | awk -F':' '{print $2}' `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Root Password Validity
    temp_last_chage=`  chage -l root | grep "Last"       | awk -F': ' '{print $2}' `
    temp_psw_expire=`  chage -l root | grep "Password e" | awk -F': ' '{print $2}' `
    temp_psw_inact=`   chage -l root | grep "Password i" | awk -F': ' '{print $2}' `
    temp_account_exp=` chage -l root | grep "Account e"  | awk -F': ' '{print $2}' `
    temp_min_change=`  chage -l root | grep "Minimum"    | awk -F': ' '{print $2}' `
    temp_max_change=`  chage -l root | grep "Maximum"    | awk -F': ' '{print $2}' `
    temp_warn_days=`   chage -l root | grep "warning"    | awk -F': ' '{print $2}' `
    temp_now=` date +%s `
    temp_remained_days=${temp_max_change}
    if [ "${temp_psw_expire}" != "never" ]
    then
        temp_root_psw_expire_day=` chage -l root | grep "Password expires" | awk -F':' '{print $2}' | xargs -I {} date -d "{}" +%s `
        let temp_remained_days=( ${temp_root_psw_expire_day} - ${temp_now} )/86400
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : password validity  ---------
Last password change                     : ${temp_last_chage}
Password expires                         : ${temp_psw_expire}
Password inactive                        : ${temp_psw_inact}
Account expires                          : ${temp_account_exp}
Minimum days between password change     : ${temp_min_change}
Maximum days between password change     : ${temp_max_change}
Days of warning before password expires  : ${temp_warn_days}
Days when root password expired          : ${temp_remained_days} days
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${temp_remained_days} -le 30 ]
    then
        echo "------  RESULT(password validity) : [ wrong ]  ------"  >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "root password expires in ${temp_remained_days} days"
    else
        echo "------  RESULT(password validity) : [ correct ]  ------">> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # System Config
    cat > ${TEMP_FILE} << EOF

------  checklist : system config  -------------
/etc/sysctl.conf :
` cat /etc/sysctl.conf | sed "/^#/d" | sed "/^$/d" `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Inodes Space
    cat > ${TEMP_FILE} << EOF

------  checklist : inodes space  --------------
` df -hi 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    check_inodes=` df -hi 2>/dev/null | grep -e "[8-9][0-9]%\|100%" `
    if [ -n "${check_inodes}" ]
    then
        echo "------  RESULT(inodes space)>80% : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "inodes usage is higher then 80%"
    else
        echo "------  RESULT(inodes space) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Memory Use
    cat > ${TEMP_FILE} << EOF

------  checklist : memory use  ----------------
( unit : MB )
` free -m `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Memory Detail
    cat > ${TEMP_FILE} << EOF

------  checklist : memory detail  -------------
（按系统统计）：
` ${SHELL_DIR}/smem-1.4/smem -c "area used cache noncache" -w -k 2>/dev/null `

（按用户统计 RSS降序）:
` ${SHELL_DIR}/smem-1.4/smem -c "user count swap uss pss rss" -u -k -s rss -r 2>/dev/null | head -20 `

（按进程统计 RSS降序）：
` ${SHELL_DIR}/smem-1.4/smem -c "pid user command swap uss pss rss" -k -s rss -r 2>/dev/null | head -20 `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Swap Use
    cat > ${TEMP_FILE} << EOF

------  checklist : swap use  ------------------
( unit : KB )
` ${SHELL_DIR}/smem-1.4/smem -c "pid user command swap uss pss rss" -k -s swap -r 2>/dev/null | head -10 `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ 1 -lt ` ${SHELL_DIR}/smem-1.4/smem -c "swap" -s swap -r 2>/dev/null | head -10 | grep -v -e " 0 " | wc -l ` ]
    then
        echo "------  RESULT(swap use) : [ wrong ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "memory is not enough"
    else
        echo "------  RESULT(swap use) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Auto-Start
    cat > ${TEMP_FILE} << EOF

------  checklist : auto-start  ----------------
` cat ${AUTO_START_CONFIG} | grep -v "^$" `

` chkconfig --list corosync 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    temp_ha=` grep -v "^#" ${AUTO_START_CONFIG} | grep -e "haChkconfig.sh" -e "haservice.sh" -e "n1service.sh" -e "n+1service.sh" `
    if [ ""X == "${temp_ha}"X ]
    then
        if [ "${CentOS_VER}" -eq 6 ]
        then
            temp_ha=` chkconfig --list | grep corosync | grep -e '2:on'$'\t''3:on'$'\t''4:on'$'\t''5:on' 2>/dev/null `
            if [ ""X == "${temp_ha}"X ]
            then
                check_ha="off"
            else
                check_ha="on"
            fi
        else
            check_ha="off"
        fi
    else
        check_ha="on"
    fi
    # Other Auto-Start
    temp_other=` grep -v "^#" ${AUTO_START_CONFIG} | grep -v "haChkconfig.sh" | grep -v "haservice.sh" | grep -v "n1service.sh" | grep -v "n+1service.sh" | grep -e "start" | grep -e "server.sh" `
    if [ ""X == "${temp_other}"X ]
    then
        check_other="off"
    else
        check_other="on"
    fi
    if [ "on"X == "${check_ha}"X ] && [ "on"X == "${check_other}"X ]
    then
        check_result="[ wrong ]"
    else
        check_result="[ correct ]"
    fi
    cat >> ${CHECK_FILE} << EOF
--------------------
HA or N+1 : ${check_ha}
Other     : ${check_other}
------  RESULT(auto-start) : ${check_result}  ------
EOF
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_result}" "HA or N+1 : ${check_ha}
Other     : ${check_other}"


    my_wait # Crontab
    cat > ${TEMP_FILE} << EOF

------  checklist : system timing job  ---------
/etc/crontab :
` sed -n '/ \* /p' /etc/crontab `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ "${CentOS_VER}" -eq 7 ]
    then
        check_crond=` systemctl status crond.service | grep " active (running) " `
    else
        check_crond=` service crond status | grep " is running" `
    fi
    if [ -z "${check_crond}" ]
    then
        echo "------  RESULT(crond running) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "crond running" "[ wrong ]"   "crond.service not running"
    else
        echo "------  RESULT(crond running) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "crond running" "[ correct ]" ""
    fi
    
    temp_dbbk=` grep "/usr/local/bin/dbbk.sh" /etc/crontab | wc -l `
    if [ ${temp_dbbk} -gt 1 ]
    then
        echo "------  RESULT(repeated dbbk) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "repeated dbbk.sh"
    else
        echo "------  RESULT(repeated dbbk) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi
    
    temp_pgrman=` grep "pg_rman_backup.sh" /etc/crontab| wc -l `
    if [ ${temp_pgrman} -gt 0 ]
    then
        echo "------  RESULT(pgsql pg_rman) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "pgsql pg_rman" "[ wrong ]"   "suggest to uninstall pg_rman"
    else
        echo "------  RESULT(pgsql pg_rman) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "pgsql pg_rman" "[ correct ]" ""
    fi


    my_wait # Source Limited
    cat > ${TEMP_FILE} << EOF

------  checklist : source limited  ------------
ulimit :
` ulimit -a `
--------------------
/etc/security/limits.d/nproc.conf :
` cat /etc/security/limits.d/*nproc.conf `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Imos Install
    cat > ${TEMP_FILE} << EOF

------  checklist : imos install  --------------
install pdt : ` ls /usr/local/sbin/services/ 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Linux Audit
    check_try_stop="failed"
    check_audit="off"
    [ ` auditctl -s | grep -e "enabled=1" -e "enabled 1" | wc -l ` -eq 1 ] && check_audit="on"
    # try to stop & disable auditd
    if [ "${CentOS_VER}" -eq 7 ]
    then
        service auditd stop >/dev/null 2>&1 && systemctl disable auditd >/dev/null 2>&1
        rc=$?
    else
        service auditd stop >/dev/null 2>&1 && chkconfig auditd off >/dev/null 2>&1
        rc=$?
    fi
    [ ${rc} -eq 0 ] && check_try_stop="ok"
    cat > ${TEMP_FILE} << EOF

------  checklist : linux audit  ---------------
linux audit : ${check_audit}
try disable : ${check_try_stop}
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Linux Dmesg
    cat >> ${CHECK_FILE} << EOF

------  checklist : linux dmesg  ---------------
EOF
    dmesg -V > /dev/null 2>&1
    # date -d "1970-01-01 UTC `echo "$(date +%s)-$(cat /proc/uptime|cut -f 1 -d' ')+203869.799656"|bc `seconds"
    if [ $? -eq 0 ]
    then
        dmesg -T | grep -v "^$" | tail -100 >> ${CHECK_FILE}
    else
        dmesg    | grep -v "^$" | tail -100 >> ${CHECK_FILE}
    fi
    Write_to_CHECK_CSV "------  checklist : linux dmesg  ---------------" "" "See details in check.log"
	
	
# 查询BIOS资源冲突相关文件（/proc/cmdline） ENVmonitor -g -hwver执行结果Main_Board为VS01VMCUA说明存在问题。
	if ENVmonitor -g -hwver &> /dev/null; then
		my_wait # DB Table error data
		cat > ${TEMP_FILE} << EOF
------  checklist : error of BIOS resource conflicts  -----------

EOF
		cat ${TEMP_FILE} >> ${CHECK_FILE}
		Main_Board=`ENVmonitor -g -hwver | grep '^Main_Board[[:space:]]\+[^[:space:]]' | awk '{print$2}'`
		if [ "$Main_Board" == "VS01VMCUA" ]; then
			more /proc/cmdline | grep "acpi_enforce_resources=no"  &> /dev/null
			if [ $? -eq 0 ]; then
				echo "------  RESULT(BIOS resource) : [ correct ]  ------" >> ${CHECK_FILE}
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
			else
				echo "------  RESULT(BIOS resource) : [ wrong ]  ------" >> ${CHECK_FILE}
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
			fi
		else
			echo "------  RESULT(BIOS resource) : [ correct ]  ------" >> ${CHECK_FILE}
			Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
		fi
	else
		cat > ${TEMP_FILE} << EOF
------  checklist : error of BIOS resource conflicts  -----------

------  RESULT(BIOS resource) : [ correct ]  ------
EOF
		cat ${TEMP_FILE} >> ${CHECK_FILE}
		Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
	fi	
    echo "================================================" >> ${CHECK_FILE}
	
	
	# 查询messages最近150条使用sftp执行的操作
	if [ -f /var/log/messages ];then
		my_wait # DB Table error data
		cat > ${TEMP_FILE} << EOF
------  checklist : Mesag on the operation of Sftp (Last 150)  -----------
------  info begin

EOF
		cat ${TEMP_FILE} >> ${CHECK_FILE}
		more /var/log/messages | grep sftp-server | grep -E "rename|open" | tail -150 >> ${CHECK_FILE}
		echo "" >> ${CHECK_FILE}
		echo "------  info end" >> ${CHECK_FILE}
	fi
}

Process_Status()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : Process Status
EOF


    my_wait 0 # Process Status
    top -o %MEM -n 1 >/dev/null 2>&1
    if [ $? -eq 0 ] # top enable sort
    then
        cat > ${TEMP_FILE} << EOF

------  checklist : process (top)  -------------
sort by %CPU:
` top -c -b -o %CPU -n 1 | head -17 `
--------------------
sort by %MEM:
` top -c -b -o %MEM -n 1 | head -17 `
EOF
    else
        cat > ${TEMP_FILE} << EOF

------  checklist : process (top)  -------------
` top -c -b -n 1 | head -17 `
EOF
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Abnormal Process
    check_zombie=` ps axfj | grep -w "<defunct>" | grep -v "grep" `
    check_num1=`   ps axfj | grep -w "<defunct>" | grep -v "grep" | wc -l `
    check_tcpdump=`ps axfj | grep -w "tcpdump"   | grep -v "grep" `
    check_num2=`   ps axfj | grep -w "tcpdump"   | grep -v "grep" | wc -l `
    cat > ${TEMP_FILE} << EOF

------  checklist : abnormal process  ----------
EOF
    [ ! -z "${check_zombie}"  ] && echo "${check_zombie}"  >> ${TEMP_FILE}
    [ ! -z "${check_tcpdump}" ] && echo "${check_tcpdump}" >> ${TEMP_FILE}
    # zombie process check
    if [ "${check_num1}" -ne 0 ]
    then
        echo "------  RESULT(zombie process)=${check_num1} : [ wrong ]  ------"   >> ${TEMP_FILE}
    else
        echo "------  RESULT(zombie process)=${check_num1} : [ correct ]  ------" >> ${TEMP_FILE}
    fi

    # tcpdump process check
    if [ "${check_num2}" -ne 0 ]
    then
        echo "------  RESULT(tcpdump process)=${check_num2} : [ wrong ]  ------"   >> ${TEMP_FILE}
    else
        echo "------  RESULT(tcpdump process)=${check_num2} : [ correct ]  ------" >> ${TEMP_FILE}
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ "${check_num1}" -ne 0 -o "${check_num2}" -ne 0 ]
    then
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "exists zombie process or tcpdump"
    else
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Imos Process
    cat > ${TEMP_FILE} << EOF

------  checklist : imos process  --------------
START TIME               PROCESS
` ps -A -o lstart,cmd | grep -e "/usr/local/bin/" | grep -v "imosproxy" | grep -v grep | sort -n -k 1,5 -t ' ' `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    # my_wait # lib.so
    # lib_array=(
    # "/home/postgres/pgsql/lib/postgis-2.4.so"
    # "/home/postgres/pgsql/lib/rtprostgis-2.4.so"
    # )
    # for item in ${lib_array[@]}
    # do
    #     echo "check ${item} :"
    #     ldd "${item}" | grep "=>" | awk '{print $1,$2,$3" ->"}' | grep -v "(" | column -t > ${TEMP_FILE}
    #     while read line
    #     do
    #         lib_file=` echo "${line}"     | awk -F'=>'  '{print $2}' | awk '{print $1}' `
    #         lib_real=` ls -l ${lib_file}  | awk -F'->'  '{if( $2 ~ /so/ ) print $2 ; else print " not-link-file"}' `
    #         lib_elf=` file -L ${lib_file} | awk -F':|,' '{print $2}' | awk '{print $1,$2,$3}'`
    #         echo -e "${line} ${lib_real} -- ${lib_elf}"
    #     done < ${TEMP_FILE} | column -t
    #     echo
    # done

    echo "================================================" >> ${CHECK_FILE}
}

Disk_Space()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : Disk Space
EOF


    my_wait 0 # Fstab
    cat > ${TEMP_FILE} << EOF

------  checklist : fstab config  --------------
/etc/fstab :
` sed "/^$/d" /etc/fstab | sed "/^#/d" `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Disk Space
    cat > ${TEMP_FILE} << EOF

------  checklist : disk space  ----------------
` df -lTh 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    check_space=` df -lhT 2>/dev/null | grep -e "[8-9][0-9]%\|100%" `
    if [ -n "${check_space}" ]
    then
        echo "------  RESULT(disk space)>80% : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "disk usage is higher then 80%"
    else
        echo "------  RESULT(disk space) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    # Dir Mod & Dir Space
    for ((num=0 ; num < ${#dir_array[@]} ; num++))
    do
        my_wait # Status of Dir
        if [ -e "${dir_array[num]}" ]
        then
            temp_mod=` ls -ld ${dir_array[num]} | awk -F' ' '{print $1","$3","$4}' `
            temp_mod=${temp_mod//./}
            # temp_mod=${check_dir:0:10}
            cat > ${TEMP_FILE} << EOF

------  checklist : dir state  -----------------
Dir Check   : ${dir_array[num]}
Dir Mod     : ${temp_mod}
Correct Mod : ${dir_mod_array[num]}
Dir Space   : ` du -sh ${dir_array[num]} 2>/dev/null | awk -F' ' '{print $1}'`
EOF
            if [ "${temp_mod}"X == "${dir_mod_array[num]}"X ]
            then
                check_mod="[ correct ]"
            elif [ ! -f /usr/local/imosconfig/vmconf/vmsys.conf ] && [ "${dir_array[num]}"X == "/mnt/syncdata/resftp"X ] && [ "${temp_mod}"X == "drwxr-xr-x,root,root"X ]
            then # if not VM , mod is drwxr-xr-x,root,root
                check_mod="[ correct ]"
            else
                check_mod="[ wrong ]"
            fi
            cat ${TEMP_FILE} >> ${CHECK_FILE}
            echo "------  RESULT(dir mod) : ${check_mod}  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_mod}" ""
        fi
    done


    my_wait # Large File
    check_num_1=` find /var/log -type f -size +2G | wc -l `
    check_num_2=` find /root/core* -type f -size +2G 2>/dev/null | wc -l `
    check_num_3=` find /var/lib/pacemaker/cores/ -type f -size +2G 2>/dev/null | wc -l `
    check_num=$((check_num_1 + check_num_2 + check_num_3))
    cat > ${TEMP_FILE} << EOF

------  checklist : large file  ----------------
( /var/log  /root/core*  /var/lib/pacemaker/cores/ )
file size > 2G :
EOF
    [ 0 -ne ${check_num_1} ] && find /var/log                  -type f -size +2G 2>/dev/null | xargs -I {} ls -lh {} >> ${TEMP_FILE}
    [ 0 -ne ${check_num_2} ] && find /root/core*               -type f -size +2G 2>/dev/null | xargs -I {} ls -lh {} >> ${TEMP_FILE}
    [ 0 -ne ${check_num_3} ] && find /var/lib/pacemaker/cores/ -type f -size +2G 2>/dev/null | xargs -I {} ls -lh {} >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${check_num} -gt 0 ]
    then
        echo "------  RESULT(large file) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
    else
        echo "------  RESULT(large file) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Lsof
    timeout 20 lsof -s >/tmp/GeneralCheck_lsof.txt 2>/dev/null
    if [ $? -eq 0 ]
    then   # lsof doesn't run timeout
        grep -e "FD" -e "(deleted)" /tmp/GeneralCheck_lsof.txt >/tmp/GeneralCheck_lsof2.txt
        check_num=` grep -c -e "FD" -e "(deleted)" /tmp/GeneralCheck_lsof.txt `
        check_space=` sed -s '1d' /tmp/GeneralCheck_lsof2.txt | awk '{s+=$7}END{print s/1024/1024}' | awk -F'.' '{print $1}' `
        cat > ${TEMP_FILE} << EOF

------  checklist : lsof  ----------------------
file deleted but unreleased :
count : ${check_num}
space : ${check_space} M
EOF
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        if [ ${check_space} -gt 5120 ]
        then
            echo "------  RESULT(deleted but unreleased) : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "deleted but unreleased space : ${check_space} M"
        else
            echo "------  RESULT(deleted but unreleased) : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" "deleted but unreleased space : ${check_space} M"
        fi
    else
        cat > ${TEMP_FILE} << EOF

------  checklist : lsof  ----------------------
command(lsof -s) run timeout, skip
EOF
        cat ${TEMP_FILE} >> ${CHECK_FILE}
    fi
    rm -rf /tmp/GeneralCheck_lsof.txt  2>/dev/null
    rm -rf /tmp/GeneralCheck_lsof2.txt 2>/dev/null


    my_wait # Core File
    cat > ${TEMP_FILE} << EOF

------  checklist : core file  -----------------
EOF
    core_flag=0
    if [ -d "${HA_CORE_DIR}" ]
    then
        core_file=(` ls -dt ${HA_CORE_DIR}* 2>/dev/null | head -10 `)
        for file in ${core_file[@]}
        do
            temp_core=""
            core_flag=1
            ls -lht --time-style='+%Y-%m-%d %H:%M:%S' ${file} | awk -F' ' '{print $8,$5,$6,$7}'
            temp_core=` find ${file} -type f -exec file '{}' \; `
            echo -e "${temp_core#*:} \n"
        done >> ${TEMP_FILE}
    fi

    core_file=(` ls -dt /root/core* 2>/dev/null | head -10 `)
    for file in ${core_file[@]}
    do
        temp_core=""
        core_flag=1
        ls -lht --time-style='+%Y-%m-%d %H:%M:%S' ${file} | awk -F' ' '{print $8,$5,$6,$7}'
        temp_core=` find ${file} -type f -exec file '{}' \; `
        echo -e "${temp_core#*:} \n"
    done >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
    [ ${core_flag} -eq 0 ] && echo >> ${CHECK_FILE}


    my_wait # Core Analysis
    echo "------  checklist : core analysis  -------------" >> ${CHECK_FILE}
    Core_Analysis
    echo "================================================" >> ${CHECK_FILE}
}

Network_Status()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : Network Status
EOF


    my_wait 0 # Network Card, Routing
    cat > ${TEMP_FILE} << EOF

------  checklist : network card  --------------
` ifconfig `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
    ip_card_array=(` ifconfig | grep -v '^ ' | grep -v "^$" | grep -v "lo" | awk -F' ' '{print $1}' `)
    for ip_card in ${ip_card_array[@]}
    do
        temp_num=` ethtool -S ${ip_card} 2>/dev/null | grep "error" | grep -v " 0" | wc -l `
        if [ "${temp_num}" -ne 0 ]
        then
            echo "${ip_card}:"                                              >> ${CHECK_FILE}
            ethtool -S ${ip_card} 2>/dev/null | grep "error" | grep -v " 0" >> ${CHECK_FILE}
        fi
    done


    my_wait # Netcard Speed
    cat > ${TEMP_FILE} << EOF

------  checklist : netcard speed  -------------
EOF
    check_speed=0
    temp_card_speed=` sar -n DEV 1 1 2>/dev/null | grep "Average" `
    if [ -z "${temp_card_speed}" ]
    then
        echo "unknown" >> ${TEMP_FILE}
    else
        ip_card_array=(` ifconfig -s | grep -v ":" | awk '{print $1}' | grep -v Iface `)
        
        for card in ${ip_card_array[@]}
        do
            temp_rx=` echo -e "${temp_card_speed}" | grep -w "${card}" | awk '{print $5}' `
            temp_tx=` echo -e "${temp_card_speed}" | grep -w "${card}" | awk '{print $6}' `
            speed=`   ethtool ${card}              | grep "Speed"      | tr -cd "[0-9]" `
            if [ -n "${temp_rx}" ] && [ -n "${temp_tx}" ]
            then
                if [ "lo"X != "${card}"X ]
                then
                    echo -e "${card} : ${speed} MB/s"
                else
                    echo -e "${card} : "
                fi
                echo -e "    recieve_flow  : ${temp_rx} KB/s"
                echo -e "    transfer_flow : ${temp_tx} KB/s"
                if [ "lo"X != "${card}"X ] && [ -n "${speed}" ] && [ ${speed} -lt 1000 ]
                then
                    check_speed=1
                fi
            fi
        done >> ${TEMP_FILE}
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${check_speed} -eq 1 ]
    then
        echo "------  RESULT(netcard speed)<1000M/s : [ wrong ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "Net Speed < 1000M/s"
    else
        echo "------  RESULT(netcard speed) : [ correct ]  ------"         >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # IP Route
    cat > ${TEMP_FILE} << EOF

------  checklist : ip route  ------------------
` ip route `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""

    # if [ "${CentOS_VER}" -eq 7 ]
    # then
    #     ip_mask=` ifconfig | grep -w "${IP_INFO}" | awk -F"mask" '{print $2}' | awk '{print $1}' `
    # else
    #     ip_mask=` ifconfig | grep -w "${IP_INFO}" | awk -F"Mask:" '{print $2}' `
    # fi
    # temp_mask=""
    # for ip in ` echo ${ip_mask} | awk -F'.' '{print $1,$2,$3,$4}' `
    # do
    #     temp_mask=${temp_mask}"` echo "obase=2;ibase=10;${ip}" | bc 2>/dev/null | xargs printf "%08d\n" `"
    #     # echo ${temp_mask}
    # done
    # countNum1=` echo ${temp_mask} | awk -F'1' '{print NF-1}' `
    # countNum0=` echo ${temp_mask} | awk -F'0' '{print NF-1}' `
    # temp_count_subnet=""
    # for (( i=0 ; i<${countNum0} ; i++ ))
    # do
    #     temp_count_subnet=${temp_count_subnet}"1"
    #     # echo ${temp_count_subnet}
    # done
    # check_count_subnet=` echo "obase=10;ibase=2;${temp_count_subnet}" | bc 2>/dev/null `


    my_wait # ARP Para
    gc_interval="/proc/sys/net/ipv4/neigh/default/gc_interval"
    gc_stale_time="/proc/sys/net/ipv4/neigh/default/gc_stale_time"
    gc_thresh1="/proc/sys/net/ipv4/neigh/default/gc_thresh1"
    gc_thresh2="/proc/sys/net/ipv4/neigh/default/gc_thresh2"
    gc_thresh3="/proc/sys/net/ipv4/neigh/default/gc_thresh3"
    check_count=` [ -f ${gc_thresh1} ] && cat ${gc_thresh1} || echo "0" `
    temp_count=` timeout 20 arp -a 2>/dev/null | wc -l || echo 0 `
    cat > ${TEMP_FILE} << EOF

------  checklist : arp config  ----------------
gc_interval   : ` [ -f ${gc_interval} ] && cat ${gc_interval} `
gc_stale_time : ` [ -f ${gc_stale_time} ] && cat ${gc_stale_time} ` `echo -n -e "\t"`(default 60)
gc_thresh1    : ${check_count} `echo -n -e "\t"`(default 128)
gc_thresh2    : ` [ -f ${gc_thresh2} ] && cat ${gc_thresh2} ` `echo -n -e "\t"`(default 512)
gc_thresh3    : ` [ -f ${gc_thresh3} ] && cat ${gc_thresh3} ` `echo -n -e "\t"`(default 1024)
Count of ARPs : ${temp_count}
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${temp_count} -eq 0 ]
    then
        echo "command(arp -a) run timeout, skip"               >> ${CHECK_FILE}
        echo "------  RESULT(arp count) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "command(arp -a) run timeout, skip"
    elif [ ${temp_count} -gt ${check_count} ]
    then
        echo "------  RESULT(arp count) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "${temp_count} > gc_thresh1(${check_count})"
    else
        echo "------  RESULT(arp count) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # IP Conflict
    ip_card=` ifconfig | grep -B1 "${IP_INFO}" | head -1 | awk -F ' |:' '{print $1}' `
    cat > ${TEMP_FILE} << EOF

------  checklist : ip conflict  ---------------
` arping ${IP_INFO} -D -I ${ip_card} -w 3 `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    arping ${IP_INFO} -D -I ${ip_card} -w 3 > /dev/null 2>&1
    check_conflict=$?
    if [ "${check_conflict}" -eq 0 ]
    then
        echo "------  RESULT(ip conflict) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    else
        echo "------  RESULT(ip conflict) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "detect a same ip address in this network"
    fi


    my_wait # Ping Gateway
    check_mod="[ correct ]"
    cat > ${TEMP_FILE} << EOF

------  checklist : ping gateway  --------------
EOF

    if [ -s "/var/log/cluster/ping.log" ] # if exist and not empty
    then
        # check ping.log
        echo "ping.log : exist" >> ${TEMP_FILE}
        grep -e "packet loss" /var/log/cluster/ping.log | grep -v " 0% packet loss" | tail -30 >> ${TEMP_FILE}
        check_ping=` grep -e "packet loss" /var/log/cluster/ping.log | grep -v " 0% packet loss" | wc -l `
        if [ ${check_ping} -gt 0 ]
        then
            check_mod="[ wrong ]"
        fi
    else
        # Ping Test in Manual
        echo -e "ping.log : non-exist or empty\n" >> ${TEMP_FILE}
        if [ -z "${GATEWAY}" ]
        then
            echo "--[error]-- Ping Gateway : Can't get gateway ip " >> ${TEMP_FILE}
            check_mod="[ wrong ]"
        else
            temp_time=1
            while (( "${temp_time} < 3" ))
            do
                echo "Group ${temp_time} Ping Test :" >> ${TEMP_FILE}
                ping -n -W 1 -c 10 ${GATEWAY}         >> ${TEMP_FILE} 2>/dev/null
                echo                                  >> ${TEMP_FILE}
                my_wait # Ping Test
                let "temp_time+=1"
            done
            if [ ` grep -e "packet loss" ${TEMP_FILE} | grep -c -v " 0% packet loss" ` -ne 0 ]
            then
                check_mod="[ wrong ]"
            fi
        fi
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    echo "------  RESULT(ping gateway) : ${check_mod}  ------" >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_mod}" ""


    my_wait # Selinux
    cat > ${TEMP_FILE} << EOF

------  checklist : selinux  -------------------
` cat /etc/selinux/config 2>/dev/null | grep "^SELINUX=" `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    # normally it should be 'SELINUX=disabled', but found one like 'SELINUX=disable'
    if [ -z "` cat /etc/selinux/config 2>/dev/null | grep "^SELINUX=disable" `" ]
    then
        echo "------  RESULT(selinux) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
    else
        echo "------  RESULT(selinux) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Firewall
    if [ "${CentOS_VER}" -eq 7 ]
    then
        cat > ${TEMP_FILE} << EOF

------  checklist : firewall  ------------------
` systemctl status firewalld `
EOF
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        check_firewall=` systemctl status firewalld | grep "Active: inactive (dead)" `
        if [ -z "${check_firewall}" ]
        then
            echo "------  RESULT(firewall running) : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
        else
            echo "------  RESULT(firewall running) : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi
    else
        check_start=` chkconfig --list | grep iptables | grep -e '2:on'$'\t''3:on'$'\t''4:on'$'\t''5:on' 2>/dev/null `
        cat > ${TEMP_FILE} << EOF

------  checklist : iptables  ------------------
auto_start:
`chkconfig --list | grep iptables `
rule:
` service iptables status `
EOF
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        if [ ! -z "${check_start}" ]
        then
            echo "------  RESULT(iptables auto-start) : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "${check_start}"
        else
            echo "------  RESULT(iptables auto-start) : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi
    fi


    my_wait # SSH Limitation
    cat > ${TEMP_FILE} << EOF

------  checklist : sshd limit  ----------------
Allow :
` sed "/^#/d" /etc/hosts.allow `
Deny :
` sed "/^#/d" /etc/hosts.deny `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # SSH Config
    cat > ${TEMP_FILE} << EOF

------  checklist : sshd config  ---------------
` grep -w -e "Port" -e "PermitRootLogin" /etc/ssh/sshd_config `

------  checklist : ssh config  ----------------
` grep -w -e "Port" -e "Cipher" -e "Ciphers" /etc/ssh/ssh_config `
================================================
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
}

PG_Status()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : Postgresql Status
EOF


    my_wait 0 # Basic Info
    if [ -f "${IMOS_CFG}" ]
    then
        temp_pg_host=` grep "DBServerName" ${IMOS_CFG} 2>/dev/null `
        temp_pg_host=${temp_pg_host##*=}
        if [ -n "${temp_pg_host}" ]
        then
            DB_PGPORT=` echo ${temp_pg_host} | awk -F':' '{print $2}' `
            DB_PGHOST=` echo ${temp_pg_host} | awk -F':' '{print $1}' `
        fi
    else
        echo -e "\n****** imos.cfg not find, skip ******"       >> ${CHECK_FILE}
        echo "================================================" >> ${CHECK_FILE}
        return
    fi
    pg_psql_para="-U ${DB_USER} -h ${DB_PGHOST} -p ${DB_PGPORT}"
    # get DB password
    if [ -f "/usr/local/bin/common.sh" ]
    then
        source /usr/local/bin/common.sh
        export_dbpasswd
        [ $? -ne 0 ] && Get_DBpasswd
    else
        Get_DBpasswd
    fi
    DB_VERSION=` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SELECT version();" 2>/dev/null | awk -F' ' '{print $1,$2}' `
    temp_version=` echo "${DB_VERSION}" | awk -F' ' '{print $2}' `
    temp_flag=` ifconfig | grep "${DB_PGHOST}" | wc -l `
    [ "${temp_flag}" -eq 0 ] && check_flag="Yes" || check_flag="No"
    cat > ${TEMP_FILE} << EOF

------  checklist : pg basic info  -------------
DB Server Ip   : ${DB_PGHOST}
DB Server Port : ${DB_PGPORT}
DB Version     : ${DB_VERSION}
Independent DB : ${check_flag}
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""

    if [ ""X == "${DB_VERSION}"X ]
    then
        echo -e "\n****** DB not running, skip ******"          >> ${CHECK_FILE}
        echo "================================================" >> ${CHECK_FILE}
        return
    fi


    my_wait # Pg Config
    temp_shared_buffers=` free -g | grep "Mem:" | awk -F' ' '{printf("max %2.0fGB",$2/8)}' `
    cat > ${TEMP_FILE} << EOF

------  checklist : pg config  -----------------
timezone                : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW timezone;"` (Asia/Shanghai)
log_timezone            : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW log_timezone;"` (Asia/Shanghai)
unix_socket_directories : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW unix_socket_directories;"` `echo -n -e "\t\t"`(/tmp)
client_encoding         : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW client_encoding;"` `echo -n -e "\t\t"`(UTF8)
fsync                   : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW fsync;"` `echo -n -e "\t\t"`(on)
max_connections         : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW max_connections;"` `echo -n -e "\t\t"`(1200)
synchronous_commit      : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW synchronous_commit;"` `echo -n -e "\t\t"`(off)
autovacuum              : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW autovacuum;"` `echo -n -e "\t\t"`(on)
track_counts            : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW track_counts;"` `echo -n -e "\t\t"`(on)
shared_buffers          : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW shared_buffers;" ` `echo -n -e "\t\t"`(${temp_shared_buffers})
effective_cache_size    : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW effective_cache_size;" `
work_mem                : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW work_mem;" `
maintenance_work_mem    : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW maintenance_work_mem;" `
log_statement           : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW log_statement;" `
log_lock_waits          : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW log_lock_waits;" `
deadlock_timeout        : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW deadlock_timeout;" `
archive_mode            : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW archive_mode;" `
archive_command         : ` ${MY_PGSQL} ${pg_psql_para} -qAt -c "SHOW archive_command;" `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Login Password
    temp_pg_psw=` ${MY_PGSQL} ${pg_psql_para} -qAt -c "
    SELECT count(1) FROM pg_user WHERE valuntil <> 'infinity' AND valuntil IS NOT NULL;" `
    cat > ${TEMP_FILE} << EOF

------  checklist : pg psw life  ---------------
EOF
    ${MY_PGSQL} ${pg_psql_para} -c "SELECT * FROM pg_user;" >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${temp_pg_psw} -ne 0 ]
    then
        echo "------  RESULT(pg psw life) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
    else
        echo "------  RESULT(pg psw life) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Storage Space
    cat > ${TEMP_FILE} << EOF

------  checklist : pg storage space  ----------
EOF
    ${MY_PGSQL} ${pg_psql_para} -c "
    SELECT d.datname AS Name, pg_catalog.pg_get_userbyid(d.datdba) AS Owner, 
        CASE WHEN pg_catalog.has_database_privilege(d.datname, 'CONNECT') 
        THEN pg_catalog.pg_size_pretty(pg_catalog.pg_database_size(d.datname)) 
        ELSE 'No Access' END AS SIZE
    FROM pg_catalog.pg_database d
    ORDER BY CASE WHEN pg_catalog.has_database_privilege(d.datname, 'CONNECT') 
        THEN pg_catalog.pg_database_size(d.datname) 
        ELSE NULL END DESC LIMIT 10;" >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Connection
    cat > ${TEMP_FILE} << EOF
------  checklist : pg connection  -------------
all connections :
EOF
    # DB connection statistics
    ${MY_PGSQL} ${pg_psql_para} -c "
    ( SELECT datname, client_addr::text, state, count(*)::text FROM pg_stat_activity 
        GROUP BY datname, client_addr, state ORDER BY datname, client_addr, state ) 
    UNION ALL 
    ( SELECT repeat('-',CASE WHEN max(length(datname))<7 THEN 7 ELSE max(length(datname)) END), 
        '------------------', repeat('-', max(length(state))), '-----' FROM pg_stat_activity ) 
    UNION ALL 
    ( SELECT 'all', 'all', 'all', count(*)::text FROM pg_stat_activity );" >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
# WITH T AS (SELECT coalesce(client_addr, '127.0.0.1') AS client_addr,1 AS num FROM pg_stat_activity)
# SELECT coalesce(host(client_addr), 'in total') AS client_addr, coalesce(sum(num), 0) AS count
# FROM T GROUP BY grouping sets((client_addr), ()) ORDER BY client_addr;


    my_wait # Sql in Query
    # SELECT procpid, start, now() - start AS lap, current_query 
    # FROM (SELECT backendid, pg_stat_get_backend_pid(S.backendid) AS procpid, 
    #         pg_stat_get_backend_activity_start(S.backendid) AS start, 
    #         pg_stat_get_backend_activity(S.backendid) AS current_query 
    #         FROM (SELECT pg_stat_get_backend_idset() AS backendid) AS S 
    #         ) AS S 
    # WHERE current_query <> '<IDLE>' AND current_query <> '<command string not enabled>' 
    # ORDER BY lap DESC;
    cat >> ${CHECK_FILE} << EOF
------  checklist : sql in query  --------------
EOF
    ${MY_PGSQL} ${pg_psql_para} -x -c "
    SELECT pid, datname, usename, client_addr, application_name, wait_event_type, wait_event, state, backend_start, 
        xact_start, xact_stay, query_start, query_stay, replace(query, chr(10), '') AS query, backend_type 
    FROM ( 
        SELECT pid, datname, usename, client_addr, application_name, wait_event_type, wait_event, state, backend_start, 
            xact_start, (now() - xact_start) AS xact_stay, query_start, (now() - query_start) AS query_stay, query, backend_type 
        FROM pg_stat_activity WHERE state != 'idle' ) AS T 
    ORDER BY query_stay DESC;" >> ${CHECK_FILE}
    Write_to_CHECK_CSV "------  checklist : sql in query  --------------" "" "See details in check.log"


    my_wait # Long Transaction
    check_time="10 second"
    check_transaction=` ${MY_PGSQL} ${pg_psql_para} -Aqt -c "
    SELECT count(*) FROM pg_stat_activity 
    WHERE state <> 'idle' AND (backend_xid IS NOT NULL OR backend_xmin IS NOT NULL) 
      AND (now() - xact_start) > INTERVAL '${check_time}' ;" `
    cat > ${TEMP_FILE} << EOF
------  checklist : long transaction  ----------
(time > ${check_time}) :
EOF
    ${MY_PGSQL} ${pg_psql_para} -o /tmp/GeneralCheck_pg.txt -c "
    SELECT datname, usename, pid, query, xact_start, (now() - xact_start) AS xact_duration, 
        query_start, (now() - query_start) AS query_duration, state 
    FROM pg_stat_activity 
    WHERE state <> 'idle' AND (backend_xid IS NOT NULL OR backend_xmin IS NOT NULL) 
      AND (now() - xact_start) > INTERVAL '${check_time}' 
    ORDER BY xact_start;"
    if [ ` wc -l /tmp/GeneralCheck_pg.txt | awk '{print $1}' ` -ge 5 ]
    then
        cat /tmp/GeneralCheck_pg.txt >> ${TEMP_FILE}
        ${MY_PGSQL} ${pg_psql_para} -c "
        SELECT name, statement, prepare_time, (now() - prepare_time) AS duration, parameter_types, from_sql 
        FROM pg_prepared_statements 
        WHERE (now() - prepare_time) > INTERVAL '${check_time}' 
        ORDER BY prepare_time;" >> ${TEMP_FILE}
    else
        echo "null" >> ${TEMP_FILE}
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}

    if [ ${check_transaction} -ne 0 ]
    then
        echo "------  RESULT(long transaction) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
    else
        echo "------  RESULT(long transaction) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


    my_wait # Replication Slot
    check_slot_num=` ${MY_PGSQL} ${pg_psql_para} -Aqt -c "SELECT count(*) FROM pg_replication_slots WHERE active='f';" `
    cat > ${TEMP_FILE} << EOF

------  checklist : replication slot  ----------
EOF
    ${MY_PGSQL} ${pg_psql_para} -o /tmp/GeneralCheck_pg.txt -c "SELECT * FROM pg_replication_slots;"
    if [ ` wc -l /tmp/GeneralCheck_pg.txt | awk '{print $1}' ` -ge 5 ]
    then
        cat /tmp/GeneralCheck_pg.txt >> ${TEMP_FILE}
    else
        echo "null" >> ${TEMP_FILE}
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ${check_slot_num} -ne 0 ]
    then
        echo "------  RESULT(replication slot) : [ wrong ]  ------"   >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
    else
        echo "------  RESULT(replication slot) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
    fi


# 查询/var/log/imoslog/pg_log/系统日志里，是否有 recovery

        if [ -d "/var/log/imoslog/pg_log/" ]
        then
			my_wait # DB Table error data
			cat > ${TEMP_FILE} << EOF
			
------  checklist : error pg_log info of database  -----------

EOF
			grep_result=$(grep -rnw '/var/log/imoslog/pg_log/' -e 'recovery' --include=*.log)
			# 检查 grep 命令的退出状态  
			if [ $? -eq 0 ]; then
				cat ${TEMP_FILE} >> ${CHECK_FILE}
				echo "------  RESULT(pg_log) : [ wrong ]  ------" >> ${CHECK_FILE}
				echo "pg_log异常,数据库可能不稳定,使用命令(grep -rnw '/var/log/imoslog/pg_log/' -e 'recovery' --include=*.log),查询结果咨询研发或二线"     >> ${CHECK_FILE} # grep 找到匹配项，所以打印"有问题" 
				echo "pg_log异常,数据库可能不稳定,使用命令(grep -rnw '/var/log/imoslog/pg_log/' -e 'recovery' --include=*.log),查询结果咨询研发或二线"     >> ${TEMP_FILE}
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
			else
				cat ${TEMP_FILE} >> ${CHECK_FILE}
				echo "------  RESULT(pg_log) : [ correct ]  ------"     >> ${CHECK_FILE} # grep 没有找到匹配项，所以打印"正常"
				sed -i '/^pg_log异常/d' "${TEMP_FILE}"
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
			fi
		fi


# 查询/var/log/imoslog/autoclean_pgap_table.log日志里是否含有 recovery

        if [ -f "/var/log/imoslog/autoclean_pgap_table.log" ]
        then
			my_wait # DB Table error data
			cat > ${TEMP_FILE} << EOF
			
------  checklist : error autoclean_pgap_table.log info of database_pgap  -----------

EOF
			grep_result=$(tail -100 /var/log/imoslog/autoclean_pgap_table.log | grep -nw -e 'recovery')
			# 检查 grep 命令的退出状态  
			if [ $? -eq 0 ]; then
				cat ${TEMP_FILE} >> ${CHECK_FILE}
				echo "------  RESULT(pg_log) : [ wrong ]  ------" >> ${CHECK_FILE}
				echo "pgap每日清理存在问题，咨询二线或者研发打补丁"     >> ${CHECK_FILE}
				echo "pgap每日清理存在问题，咨询二线或者研发打补丁"     >> ${TEMP_FILE}
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
			else
				cat ${TEMP_FILE} >> ${CHECK_FILE}
				echo "------  RESULT(pg_log) : [ correct ]  ------"     >> ${CHECK_FILE}
				sed -i '/^pgap每日清理/d' "${TEMP_FILE}"
				Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
			fi
		fi

    # get DB name, only check imos and cds
    database_name_array=(` ${MY_PGSQL} ${pg_psql_para} -qAt -c "
    SELECT datname FROM pg_catalog.pg_database WHERE datname NOT IN ('postgres','template0','template1','zabbix');" `)
    for database_name in ${database_name_array[@]}
    do
        if [ ""X == "${database_name}"X ]
        then
            echo "--[error]-- PG Status : Can't get DB database name " | tee -a ${CHECK_FILE} > /dev/null
            unset PGUSER
            unset PGPASSWORD
            echo "================================================" >> ${CHECK_FILE}
            return 1
        fi
        echo -e "\n--------  check pg database : ${database_name}  --------------" >> ${CHECK_FILE}


        my_wait # History Version
        cat > ${TEMP_FILE} << EOF
------  checklist : history version  -----------
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "SELECT * FROM tbl_version_history;" >> ${TEMP_FILE} 2>/dev/null
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Storage Space
        check_item_num=20
        cat > ${TEMP_FILE} << EOF
------  checklist : table storage space  -------
(top ${check_item_num}) :
EOF
        # get the table name of TOP storage space in the specified DB
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT nspname || '.' || relname AS relname, 
            pg_size_pretty(pg_total_relation_size(C.oid)), reltuples::integer 
        FROM pg_class C LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace) 
        WHERE nspname NOT IN ('pg_catalog', 'information_schema') 
          AND C.relkind <> 'i' AND nspname !~ '^pg_toast' 
        ORDER BY pg_total_relation_size(C.oid) DESC, C.reltuples DESC 
        LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE}          >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Sub Table Count
        cat > ${TEMP_FILE} << EOF
------  checklist : sub table count  -----------
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT inhparent::regclass AS tbl_name, count(*), 
            min(inhrelid::regclass::varchar), max(inhrelid::regclass::varchar) 
        FROM pg_inherits GROUP BY tbl_name ORDER BY count(*) DESC;" >> ${TEMP_FILE}
        # SELECT substring(relname, 1, position('2' IN relname)-2) AS table_name, 
        #        count(*) AS sub_table_num, min(relname), max(relname) 
        # FROM pg_class WHERE relname ~ '^tbl_[\w]*_[\d]'
        # GROUP BY table_name ORDER BY sub_table_num DESC;
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Age
        check_item_num=5
        cat > ${TEMP_FILE} << EOF
------  checklist : table age  -----------------
(top ${check_item_num}) :
EOF
        # get the table name of the TOP age in the specified DB
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT current_database() AS database, rolname, nspname, relkind, 
            relname, age(relfrozenxid), 2^31-age(relfrozenxid) AS age_remain 
        FROM pg_roles t1 JOIN pg_class t2 ON t1.oid=t2.relowner 
            JOIN pg_namespace t3 ON t2.relnamespace=t3.oid 
        WHERE t2.relkind IN ('t','r') 
        ORDER BY age(relfrozenxid) DESC LIMIT ${check_item_num};" >> ${TEMP_FILE} 
        cat ${TEMP_FILE} >> ${CHECK_FILE}

        temp_tbl_age=` ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -qAt -c "
        SELECT count(*) FROM pg_roles t1 
            JOIN pg_class t2 ON t1.oid=t2.relowner JOIN pg_namespace t3 ON t2.relnamespace=t3.oid 
        WHERE t2.relkind IN ('t','r') AND age(relfrozenxid) > 1500000000 ;" `
        if [ ${temp_tbl_age} -ne 0 ]
        then
            echo "------  RESULT(table age) : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "age > 1500000000"
        else
            echo "------  RESULT(table age) : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi


        my_wait # Constraints
        cat > ${TEMP_FILE} << EOF

------  checklist : pg constraints  ------------
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        (SELECT (CASE contype WHEN 'p' THEN 'primary key' 
                              WHEN 'u' THEN 'unique key' 
                              WHEN 'f' THEN 'foreign key' 
                              WHEN 'c' THEN 'check constraint' 
                              WHEN 't' THEN 'trigger constraint' 
                              WHEN 'x' THEN 'exclusion constraint' END) AS type, count(*) 
        FROM pg_constraint 
        WHERE conrelid NOT IN (SELECT inhrelid FROM pg_inherits) 
          AND conrelid::regclass::varchar LIKE 'tbl_%' 
        GROUP BY contype ORDER BY (CASE contype WHEN 'p' THEN 0 
                                                WHEN 'u' THEN 1 
                                                WHEN 'f' THEN 2 
                                                WHEN 'c' THEN 3 
                                                WHEN 't' THEN 4 
                                                WHEN 'x' THEN 5 END)) 
        UNION ALL 
        (SELECT 'index' AS type, count(*) FROM pg_index)" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Index
        check_item_num=10
        cat > ${TEMP_FILE} << EOF
------  checklist : table index count ----------
(top ${check_item_num}) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT current_database(), t1.relname, 
            pg_size_pretty(pg_relation_size(t1.oid)), t3.idx_cnt 
        FROM pg_class t1, pg_namespace t2, 
            (SELECT indrelid, count(*) AS idx_cnt FROM pg_index GROUP BY 1 HAVING count(*)>1) t3 
        WHERE t1.oid=t3.indrelid AND t1.relnamespace=t2.oid AND t1.relname not like 'pg_%' 
        ORDER BY t3.idx_cnt DESC LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Index Utilization Ratio
        check_item_num=10
        cat > ${TEMP_FILE} << EOF
------  checklist : table index utilization ----
(top ${check_item_num}) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT t2.relname, t2.indexrelname, t2.idx_scan, t2.idx_tup_read, 
            t2.idx_tup_fetch, pg_size_pretty(pg_relation_size(indexrelid)) 
        FROM pg_stat_all_tables t1, pg_stat_all_indexes t2 
        WHERE t1.relid=t2.relid AND t2.schemaname NOT IN ('pg_toast','pg_catalog') 
          AND indexrelid NOT IN (SELECT conindid FROM pg_constraint WHERE contype IN ('p','u','f')) 
          AND pg_relation_size(indexrelid)>65536 
        ORDER BY t2.idx_scan LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Index Space
        check_item_num=10
        cat > ${TEMP_FILE} << EOF
------  checklist : table index space ----------
(top ${check_item_num}) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT pg_size_pretty(pg_relation_size(t1.oid)) AS idx_size, t1.relname, t2.tablename AS tbl_name,  
            pg_size_pretty(pg_total_relation_size(t2.tablename::regclass::oid)) AS tbl_total_space 
        FROM pg_class t1, pg_indexes t2 WHERE relkind='i' AND t1.relname=t2.indexname AND t1.relname not like 'pg_%' 
        ORDER BY pg_relation_size(oid) DESC LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Index Bloat
        check_item_num=10
        cat > ${TEMP_FILE} << EOF
------  checklist : table index bloat ----------
(top ${check_item_num}) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT current_database() AS db, tablename, reltuples::bigint AS tups, 
        ----relpages::bigint AS pages, otta, 
            ROUND(CASE WHEN otta=0 OR sml.relpages=0 OR sml.relpages=otta THEN 0.0 ELSE sml.relpages/otta::numeric END,1) AS tbloat, 
        ----CASE WHEN relpages < otta THEN 0 ELSE relpages::bigint - otta END AS wastedpages, 
            CASE WHEN relpages < otta THEN 0 ELSE bs*(sml.relpages-otta)::bigint END AS wastedbytes, 
        ----CASE WHEN relpages < otta THEN '0 bytes'::text ELSE (bs*(relpages-otta))::bigint || ' bytes' END AS wastedsize, 
            iname, ituples::bigint AS itups, 
        ----ipages::bigint AS ipages, iotta, 
            ROUND(CASE WHEN iotta=0 OR ipages=0 OR ipages=iotta THEN 0.0 ELSE ipages/iotta::numeric END,1) AS ibloat, 
        ----CASE WHEN ipages < iotta THEN 0 ELSE ipages::bigint - iotta END AS wastedipages, 
            CASE WHEN ipages < iotta THEN 0 ELSE bs*(ipages-iotta) END AS wastedibytes 
        ----CASE WHEN ipages < iotta THEN '0 bytes' ELSE (bs*(ipages-iotta))::bigint || ' bytes' END AS wastedisize 
        FROM 
        (   SELECT nn.nspname AS schemaname, cc.relname AS tablename, COALESCE(cc.reltuples,0) AS reltuples, 
                COALESCE(cc.relpages,0) AS relpages, COALESCE(bs,0) AS bs, 
                COALESCE(CEIL((cc.reltuples*((datahdr+ma-(CASE WHEN datahdr%ma=0 THEN ma ELSE datahdr%ma END))+nullhdr2+4))/(bs-20::float)),0) AS otta, 
                COALESCE(c2.relname,'?') AS iname, COALESCE(c2.reltuples,0) AS ituples, COALESCE(c2.relpages,0)::bigint AS ipages, 
                COALESCE(CEIL((c2.reltuples*(datahdr-12))/(bs-20::float)),0) AS iotta 
            ----very rough approximation, assumes all cols 
            FROM pg_class cc 
            JOIN pg_namespace nn ON cc.relnamespace = nn.oid AND nn.nspname <> 'information_schema' 
            LEFT JOIN 
            (   SELECT ma, bs, foo.nspname, foo.relname, 
                    (datawidth+(hdr+ma-(CASE WHEN hdr%ma=0 THEN ma ELSE hdr%ma END)))::numeric AS datahdr, 
                    (maxfracsum*(nullhdr+ma-(CASE WHEN nullhdr%ma=0 THEN ma ELSE nullhdr%ma END))) AS nullhdr2 
                FROM 
                (   SELECT ns.nspname, tbl.relname, hdr, ma, bs, 
                        SUM((1-coalesce(null_frac,0))*coalesce(avg_width, 2048)) AS datawidth, 
                        MAX(coalesce(null_frac,0)) AS maxfracsum, 
                        hdr+(SELECT 1+count(*)/8 FROM pg_stats s2
                            WHERE null_frac <> 0 AND s2.schemaname = ns.nspname AND s2.tablename = tbl.relname
                            ) AS nullhdr
                    FROM pg_attribute att 
                    JOIN pg_class tbl ON att.attrelid = tbl.oid 
                    JOIN pg_namespace ns ON ns.oid = tbl.relnamespace 
                    LEFT JOIN pg_stats s ON s.schemaname=ns.nspname 
                    AND s.tablename = tbl.relname AND s.inherited=false AND s.attname=att.attname, 
                    (   SELECT (SELECT current_setting('block_size')::numeric) AS bs,23 AS hdr, 
                        CASE WHEN v ~ 'mingw32' OR v ~ '64-bit' THEN 8 ELSE 4 END AS ma 
                        FROM (SELECT version() AS v) AS foo 
                    ) AS constants 
                    WHERE att.attnum > 0 AND tbl.relkind='r' AND tbl.relname not like 'pg_%' GROUP BY 1,2,3,4,5 
                ) AS foo 
            ) AS rs 
            ON cc.relname = rs.relname AND nn.nspname = rs.nspname 
            LEFT JOIN pg_index i ON indrelid = cc.oid 
            LEFT JOIN pg_class c2 ON c2.oid = i.indexrelid 
        ) AS sml ORDER BY wastedibytes DESC, ibloat DESC LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # Table Bloat
        check_item_num=10
        cat > ${TEMP_FILE} << EOF
------  checklist : table bloat  ---------------
(top ${check_item_num}) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT current_database() AS db, tablename, reltuples::bigint AS tups, 
        ----relpages::bigint AS pages, otta, 
            ROUND(CASE WHEN otta=0 OR sml.relpages=0 OR sml.relpages=otta THEN 0.0 ELSE sml.relpages/otta::numeric END,1) AS tbloat, 
        ----CASE WHEN relpages < otta THEN 0 ELSE relpages::bigint - otta END AS wastedpages, 
            CASE WHEN relpages < otta THEN 0 ELSE bs*(sml.relpages-otta)::bigint END AS wastedbytes, 
        ----CASE WHEN relpages < otta THEN '0 bytes'::text ELSE (bs*(relpages-otta))::bigint || ' bytes' END AS wastedsize, 
            iname, ituples::bigint AS itups, 
        ----ipages::bigint AS ipages, iotta, 
            ROUND(CASE WHEN iotta=0 OR ipages=0 OR ipages=iotta THEN 0.0 ELSE ipages/iotta::numeric END,1) AS ibloat, 
        ----CASE WHEN ipages < iotta THEN 0 ELSE ipages::bigint - iotta END AS wastedipages, 
            CASE WHEN ipages < iotta THEN 0 ELSE bs*(ipages-iotta) END AS wastedibytes 
        ----CASE WHEN ipages < iotta THEN '0 bytes' ELSE (bs*(ipages-iotta))::bigint || ' bytes' END AS wastedisize, 
        ----CASE WHEN relpages < otta THEN 
        ----    CASE WHEN ipages < iotta THEN 0 ELSE bs*(ipages-iotta::bigint) END 
        ----ELSE CASE WHEN ipages < iotta THEN bs*(relpages-otta::bigint) 
        ----    ELSE bs*(relpages-otta::bigint + ipages-iotta::bigint) END 
        ----END AS totalwastedbytes 
        FROM 
        (   SELECT nn.nspname AS schemaname,cc.relname AS tablename,COALESCE(cc.reltuples,0) AS reltuples, 
                COALESCE(cc.relpages,0) AS relpages,COALESCE(bs,0) AS bs, 
                COALESCE(CEIL((cc.reltuples*((datahdr+ma-(CASE WHEN datahdr%ma=0 THEN ma ELSE datahdr%ma END))+nullhdr2+4))/(bs-20::float)),0) AS otta, 
                COALESCE(c2.relname,'?') AS iname, COALESCE(c2.reltuples,0) AS ituples, COALESCE(c2.relpages,0)::bigint AS ipages, 
                COALESCE(CEIL((c2.reltuples*(datahdr-12))/(bs-20::float)),0) AS iotta 
            ----very rough approximation, assumes all cols 
            FROM pg_class cc 
            JOIN pg_namespace nn ON cc.relnamespace = nn.oid AND nn.nspname <> 'information_schema' 
            LEFT JOIN 
            (   SELECT ma, bs, foo.nspname, foo.relname, 
                    (datawidth+(hdr+ma-(CASE WHEN hdr%ma=0 THEN ma ELSE hdr%ma END)))::numeric AS datahdr, 
                    (maxfracsum*(nullhdr+ma-(CASE WHEN nullhdr%ma=0 THEN ma ELSE nullhdr%ma END))) AS nullhdr2 
                FROM 
                (   SELECT ns.nspname, tbl.relname, hdr, ma, bs, 
                        SUM((1-coalesce(null_frac,0))*coalesce(avg_width, 2048)) AS datawidth, 
                        MAX(coalesce(null_frac,0)) AS maxfracsum, 
                        hdr+(SELECT 1+count(*)/8 FROM pg_stats s2 
                            WHERE null_frac <> 0 AND s2.schemaname = ns.nspname AND s2.tablename = tbl.relname 
                            ) AS nullhdr 
                    FROM pg_attribute att 
                    JOIN pg_class tbl ON att.attrelid = tbl.oid 
                    JOIN pg_namespace ns ON ns.oid = tbl.relnamespace 
                    LEFT JOIN pg_stats s ON s.schemaname=ns.nspname 
                    AND s.tablename = tbl.relname AND s.inherited=false AND s.attname=att.attname, 
                    (   SELECT (SELECT current_setting('block_size')::numeric) AS bs,23 AS hdr, 
                            CASE WHEN v ~ 'mingw32' OR v ~ '64-bit' THEN 8 ELSE 4 END AS ma 
                        FROM (SELECT version() AS v) AS foo 
                    ) AS constants 
                    WHERE att.attnum > 0 AND tbl.relkind='r' AND tbl.relname not like 'pg_%' GROUP BY 1,2,3,4,5 
                ) AS foo 
            ) AS rs 
            ON cc.relname = rs.relname AND nn.nspname = rs.nspname 
            LEFT JOIN pg_index i ON indrelid = cc.oid 
            LEFT JOIN pg_class c2 ON c2.oid = i.indexrelid 
        ) AS sml ORDER BY wastedbytes DESC,tbloat DESC LIMIT ${check_item_num};" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # DB Table Locks
        check_lock=` ${MY_PGSQL} ${pg_psql_para} -Aqt -c " SELECT count(A.granted) 
        FROM pg_locks A LEFT JOIN pg_database B ON A.database=B.oid WHERE A.granted='f';" `
        cat > ${TEMP_FILE} << EOF
------  checklist : table lock  ----------------
(all locks) :
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT A.locktype, A.mode, A.granted, B.datname, 
            A.relation::regclass AS relation, A.page, A.tuple, 
            A.virtualxid, A.transactionid, A.virtualtransaction, A.pid 
        FROM pg_locks A LEFT JOIN pg_database B ON A.database=B.oid 
        ORDER BY A.granted,( CASE mode 
            WHEN 'INVALID'                  THEN 0 
            WHEN 'AccessShareLock'          THEN 1 
            WHEN 'RowShareLock'             THEN 2 
            WHEN 'RowExclusiveLock'         THEN 3 
            WHEN 'ShareUpdateExclusiveLock' THEN 4 
            WHEN 'ShareLock'                THEN 5 
            WHEN 'ShareRowExclusiveLock'    THEN 6 
            WHEN 'ExclusiveLock'            THEN 7 
            WHEN 'AccessExclusiveLock'      THEN 8 ELSE 0 END ) DESC;" >> ${TEMP_FILE}
        #cat ${TEMP_FILE} >> ${CHECK_FILE}
        #Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""

        # dead lock
        if [[ "${temp_version}" == 9.1.* ]]
        then
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -o /tmp/GeneralCheck_pg.txt -c "
            SELECT datname, procpid, usesysid, usename, application_name, client_addr, client_port, 
                backend_start, xact_start, query_start, waiting, current_query 
            FROM pg_stat_activity WHERE waiting='t';"
        elif [[ "${temp_version}" == 9.4.* ]]
        then
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -o /tmp/GeneralCheck_pg.txt -c "
            SELECT datname, pid, usesysid, usename, application_name, client_addr, client_port, 
                backend_start, xact_start, query_start, waiting, query 
            FROM pg_stat_activity WHERE waiting='t';"
        else # from 11.* to 12.*
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -o /tmp/GeneralCheck_pg.txt -c "
            SELECT datname, pid, usesysid, usename, application_name, client_addr, client_port, 
                backend_start, xact_start, query_start, wait_event_type, wait_event, query 
            FROM pg_stat_activity WHERE wait_event_type='Lock';" 2>/dev/null
        fi
        if [ ` wc -l /tmp/GeneralCheck_pg.txt | awk '{print $1}' ` -ge 5 ]
        then
            echo "(dead locks) :" >> ${TEMP_FILE}
            cat /tmp/GeneralCheck_pg.txt >> ${TEMP_FILE}
        fi
        cat ${TEMP_FILE} >> ${CHECK_FILE}

        if [ ${check_lock} -ne 0 ]
        then
            echo "------  RESULT(table lock) : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
        else
            echo "------  RESULT(table lock) : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi


        my_wait # DB Table Hit Ratio
        cat > ${TEMP_FILE} << EOF

------  checklist : table hit ratio  -----------
EOF
        if [[ "${temp_version}" == 9.1.* ]]
        then
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
            SELECT datname, 
                round(100*(xact_rollback::numeric/(CASE WHEN xact_commit > 0 THEN xact_commit ELSE 1 END + xact_rollback)),2) 
                || ' %' AS rollback_ratio, 
                round(100*(blks_hit::numeric/(CASE WHEN blks_read>0 THEN blks_read ELSE 1 END + blks_hit)),2) 
                || ' %' AS hit_ratio, blks_read, conflicts 
            FROM pg_stat_database WHERE datname='${database_name}';" >> ${TEMP_FILE}
        elif [[ "${temp_version}" == 9.4.* ]]
        then
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
            SELECT datname, 
                round(100*(xact_rollback::numeric/(CASE WHEN xact_commit > 0 THEN xact_commit ELSE 1 END + xact_rollback)),2) 
                || ' %' AS rollback_ratio, 
                round(100*(blks_hit::numeric/(CASE WHEN blks_read>0 THEN blks_read ELSE 1 END + blks_hit)),2) 
                || ' %' AS hit_ratio, blk_read_time, blk_write_time, conflicts, deadlocks 
            FROM pg_stat_database WHERE datname='${database_name}';" >> ${TEMP_FILE}
        else    # from 11.* to 12.*
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
            SELECT datname, 
                round(100*(xact_rollback::numeric/(CASE WHEN xact_commit > 0 THEN xact_commit ELSE 1 END + xact_rollback)),2) 
                || ' %' AS rollback_ratio, 
                round(100*(blks_hit::numeric/(CASE WHEN blks_read>0 THEN blks_read ELSE 1 END + blks_hit)),2) 
                || ' %' AS hit_ratio, blk_read_time, blk_write_time, conflicts, deadlocks 
            FROM pg_stat_database WHERE datname='${database_name}';" >> ${TEMP_FILE}
        fi
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # DB Table Junk Data
        cat > ${TEMP_FILE} << EOF
------  checklist : table junk data  -----------
EOF
        ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "
        SELECT current_database(), relname, n_dead_tup FROM pg_stat_all_tables 
        WHERE n_live_tup>0 AND (n_dead_tup / n_live_tup)>0.2 AND schemaname NOT IN ('pg_toast','pg_catalog') 
        ORDER BY n_dead_tup DESC LIMIT 10;" >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
# ----pg buffer stat
# SELECT checkpoints_timed, checkpoints_req, checkpoint_write_time, checkpoint_sync_time, 
#     buffers_checkpoint, buffers_clean*8/1024||' MB' AS buffers_clean, maxwritten_clean, 
#     buffers_backend*8/1024||' MB' AS buffers_backend, buffers_backend_fsync, 
#     buffers_alloc*8/1024||' MB' AS buffers_alloc, stats_reset 
# FROM pg_stat_bgwriter;


        if [ "${database_name}" == "imos" ]
        then
            my_wait # tbl_operlog_inner
            check_operlog_id=` ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -Aqt -c "
            SELECT a.atthasdef FROM pg_class AS c, pg_attribute AS a
            WHERE c.relname='tbl_operlog_inner' AND a.attrelid=c.oid AND a.attname='operlog_id';" `
            cat > ${TEMP_FILE} << EOF
------  checklist : tbl_operlog_inner  ---------
EOF
            cat ${TEMP_FILE} >> ${CHECK_FILE}
            if [ "${check_operlog_id}" == "f" -a "${DB_VERSION}" == "PostgreSQL 11.1" ]
            then
                echo "------  RESULT(operlog_id) : [ wrong ]  ------"   >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   ""
            else
                echo "------  RESULT(operlog_id) : [ correct ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
            fi
        fi
		
# 查出imos数据库tbl_res表父资源不在的id
        if [ "${database_name}" == "imos" ]
        then
			my_wait # DB Table error data
			check_error_count=` ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -Aqt -c "select count(*) from(select res_id, parent_res_id from tbl_res where parent_res_id not in (select res_id from tbl_res) and res_id<>1) a;" `
			cat > ${TEMP_FILE} << EOF

------  checklist : error data of tbl_res  -----------
EOF
			${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "select res_id, parent_res_id from tbl_res where parent_res_id not in (select res_id from tbl_res) and res_id<>1;"  >> ${TEMP_FILE}
			cat ${TEMP_FILE} >> ${CHECK_FILE}
            if [ "${check_error_count}" == "0" ]
            then
                echo "------  RESULT(tbl_res) : [ correct ]  ------"   >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
            else
                echo "------  RESULT(tbl_res) : [ wrong ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" ""
            fi
		fi

# 查询tbl_share_info是否有脏数据

        if [ "${database_name}" == "imos" ]
        then
			tbl_share_info=` /home/postgres/pgsql/bin/psql -Upostgres -d imos -Aqt -c "select relname from pg_class where relname = 'tbl_share_info';" `
			if [ "${tbl_share_info}" != "" ]
			then
				my_wait # DB Table error data
				check_error_count=` ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -Aqt -c "select count(*) from(select count(*),inter_id from tbl_share_info group by inter_id having count(*) > 1) a;" `
				cat > ${TEMP_FILE} << EOF

------  checklist : error data of tbl_share_info  -----------
EOF
				${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "select count(*),inter_id from tbl_share_info group by inter_id having count(*) > 1;"  >> ${TEMP_FILE}
				cat ${TEMP_FILE} >> ${CHECK_FILE}
				if [ "${check_error_count}" == "0" ]
				then
					echo "------  RESULT(tbl_share_info) : [ correct ]  ------"   >> ${CHECK_FILE}
					Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]"   ""
				else
					echo "------  RESULT(tbl_share_info) : [ wrong ]  ------" >> ${CHECK_FILE}
					Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" ""
				fi
			fi
		fi
		
		
# 统计30秒内的SQL语句调用情况
        if [ "${database_name}" == "imos" ]
        then
			my_wait # DB Table error data
			${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "create extension if not exists pg_stat_statements;"  &> /dev/null
			cat > ${TEMP_FILE} << EOF

------  checklist : The last 30s pg_database SQL statements have been invoked  -----------
EOF
			${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "with 
b as ( -- begin snapshot
select clock_timestamp() as snap_time,* 
 from pg_stat_statements , pg_sleep(30) --> 30 second snapshots
),
e as ( -- end snapshot
 select clock_timestamp() as snap_time,* 
 from pg_stat_statements , pg_sleep(0) 
),
u as ( -- union all as timeseries
 select * from e union all select * from b
),
d as ( -- delta values from cumulative metrics
 select
  snap_time
  ,extract( epoch from snap_time-lag(snap_time)
   over(query_cumulative_stats)) as seconds
  ,total_time-lag(total_time)
   over(query_cumulative_stats ) as total_time
  ,calls-lag(calls)
   over(query_cumulative_stats ) as calls
  ,query
 from u 
 window query_cumulative_stats as (
  partition by userid,dbid,queryid,query order by snap_time
  )
) select -- final output
  round((calls/seconds)::numeric,2) as  \"calls/seconds\",
  round(((total_time/1000)/seconds)::numeric,1) as AAS,
  date_trunc('second',(total_time/1000)* interval '1 second')
   as total_time,
  calls as total_calls, query 
 from d where calls>0 and total_time >0 order by AAS desc fetch first 15 rows only;"  >> ${TEMP_FILE}
			cat ${TEMP_FILE} >> ${CHECK_FILE}
            ${MY_PGSQL} ${pg_psql_para} -d ${database_name} -c "drop extension if exists pg_stat_statements cascade ;"  &> /dev/null
		fi
    done
    # my_wait # Pgsql11.1 client connect memory use > 500m
    # diff_memory=500
    # check_run_time=24
    # top -c -b -n 1 | sed -e '1,7d' | grep "postgres: postgres" | awk '{if($7 ~ /m|g/) print $0}' > /home/top.txt
    # array_pid=( `cat /home/top.txt | awk '{print $1}' `)
    # array_res=( `cat /home/top.txt | awk '{print $6}' `)
    # array_shr=( `cat /home/top.txt | awk '{print $7}' `)
    # array_run=( `cat /home/top.txt | awk '{print $11}' | awk -F':' '{print $1}' `)
    # array_dif=()
    # array_pid_kill=()
    # index=0
    # for pid in ${array_pid[@]}
    # do
    #     if [[ "${array_res[$index]}" == *g ]]
    #     then
    #         array_res[$index]=` echo "${array_res[$index]}" | awk -F'[g.]' '{print $1}' `
    #         let array_res[$index]=${array_res[$index]}*1024
    #     elif [[ "${array_res[$index]}" == *m ]]
    #     then
    #         array_res[$index]=` echo "${array_res[$index]}" | awk -F'[m.]' '{print $1}' `
    #     fi

    #     if [[ "${array_shr[$index]}" == *g ]]
    #     then
    #         array_shr[$index]=` echo "${array_shr[$index]}" | awk -F'[g.]' '{print $1}' `
    #         let array_shr[$index]=${array_shr[$index]}*1024
    #     elif [[ "${array_shr[$index]}" == *m ]]
    #     then
    #         array_shr[$index]=` echo "${array_shr[$index]}" | awk -F'[m.]' '{print $1}' `
    #     fi

    #     let array_dif[${index}]=array_res[$index]-array_shr[$index]
    #     # echo -e "${pid}\t${array_res[$index]}\t${array_shr[index]}\t${array_dif[${index}]}\t${array_run[index]}"
    #     if [ "${array_dif[${index}]}" -ge ${diff_memory} ] && [ "${array_run[$index]}" -ge ${check_run_time} ]
    #     then
    #         array_pid_kill[$index]=${pid}
    #     fi
    #     let "index+=1"
    # done
    #     echo -e "${array_pid_kill[@]}"


    # 查询imos数据库中，onvif相机数量
    IPCNum_onvif_pg=` ${MY_PGSQL} ${pg_psql_para} -d imos -qAt -c "
    SELECT CASE WHEN count(*)>0 THEN count(*) ELSE 0 END AS number FROM tbl_thirdparty_ipc 
    WHERE decode_tag LIKE '%ONVIF%';" 2>/dev/null `

    # 查询imos数据库中，guobiao相机数量
    IPCNum_guobiao_pg=` ${MY_PGSQL} ${pg_psql_para} -d imos -qAt -c "
    SELECT CASE WHEN count(*)>0 THEN count(*) ELSE 0 END AS number FROM tbl_thirdparty_ipc 
    WHERE decode_tag = 'guobiao';" 2>/dev/null `

    unset PGUSER
    unset PGPASSWORD
    echo "================================================" >> ${CHECK_FILE}
}

HA_Status()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : HA & N+1
EOF


    my_wait 0 # HA or N+1 Version
    cat > ${TEMP_FILE} << EOF

------  checklist : ha basic info  -------------
EOF
    check_install="not installed"
    check_ver="not installed"
    if   [ -f "${IMOS_CONFIG_DIR}/haconf/hasys.conf" ]
    then
        check_install="HA"
        temp_version=` sed -n '/HCVERO=/'p ${IMOS_CONFIG_DIR}/haconf/hasys.conf `
    elif [ -f "${IMOS_CONFIG_DIR}/n+1conf/n+1sys.conf" ]
    then
        check_install="N+1"
        temp_version=` sed -n '/HCVERO=/'p ${IMOS_CONFIG_DIR}/n+1conf/n+1sys.conf `
    # elif [ -f "${IMOS_CONFIG_DIR}/n1conf/n1sys.conf" ]
    # then
    #     check_install="N+1"
    #     temp_version=` sed -n '/HCVERO=/'p ${IMOS_CONFIG_DIR}/n1conf/n1sys.conf `
    else
        echo "HA or N+1  : ${check_install}"                    >> ${TEMP_FILE}
        echo "****** not installed HA or N+1, skip ******"      >> ${TEMP_FILE}
        echo "================================================" >> ${TEMP_FILE}
        cat  ${TEMP_FILE}                                       >> ${CHECK_FILE}
        return
    fi
    echo "HA or N+1  : ${check_install}" >> ${TEMP_FILE}
    check_ver=${temp_version#HCVERO=}
    echo "Version    : ${check_ver}"     >> ${TEMP_FILE}
    if [ -f "${HA_PATCH_LOG}" ]
    then
        temp_patch=` tac ${HA_PATCH_LOG} | grep -e "PHVERO=${check_install}" | head -1 `
        check_patch=${temp_patch#PHVERO=}
        temp_version=${check_ver#*B}
        temp_version=${temp_version%P*}
        if [[ ${check_patch} != *${temp_version}* ]]
        then
            check_patch="not installed"
        fi
    else
        check_patch="not installed"
    fi
    echo "Patch      : ${check_patch}" >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # HA Config
    if   [[ ${check_ver} == *B3329* ]] # B3329
    then
        ha_host="/usr/local/hcha/hosts"
    elif [[ ${check_ver} == *B3335* ]] # B3335
    then
        ha_host="/usr/local/habin/hosts"
    else
        ha_host="/etc/hosts"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : hosts config  --------------
/etc/hosts :
` cat /etc/hosts `

------  checklist : ha_hosts config  -----------
${ha_host} :
` cat ${ha_host} 2>/dev/null `

------  checklist : corosync config  -----------
/etc/corosync/corosync.conf :
` cat /etc/corosync/corosync.conf `

------  checklist : network config  ------------
/etc/sysconfig/network :
` cat /etc/sysconfig/network `

------  checklist : crm config  ----------------
crm configure show :
` crm configure show 2>/dev/null | grep -A 1 -i -e "node " -e "primitive DBIP" -e "primitive " `

------  checklist : postgresql config  ---------
postgresql.conf :
` cat ${DB_CONF} | grep -w -e "wal_level" -e "hot_standby" -e "max_wal_senders" -e "wal_keep_segments" -e "archive_mode" -e "archive_command" 2> /dev/null `

------  checklist : pg_hba config  -------------
pg_hba.conf :
` cat ${DB_PGHBA_CONF} 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    sed -i "s/\"/\'/g" ${TEMP_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # DB Data Dir
    rm -rf ${TEMP_FILE} 2>/dev/null
    is_Rsync
    temp_rsync=$?
    if [ ${temp_rsync} -eq 3 ] # if drbd
    then
        cat > ${TEMP_FILE} << EOF

------  checklist : drbd config  ---------------
/etc/drbd.conf :
` cat /etc/drbd.conf 2>/dev/null `
EOF
    fi

    cat >> ${TEMP_FILE} << EOF

------  checklist : pg data folder  ------------
EOF
    if [ -d "/mnt/syncdata/pgsql/data/" ]
    then
        echo "DB Data Dir : mounted" >> ${TEMP_FILE}
        df -lTh /mnt/syncdata        >> ${TEMP_FILE}
    else
        if [ ${temp_rsync} -eq 3 ] # if drbd
        then
            echo "DB Data Dir : unmounted" >> ${TEMP_FILE}
            lvscan                         >> ${TEMP_FILE}
        fi
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Status of HA or N+1
    temp_corosync_ver=` rpm -qa corosync  | head -1 | awk -F'-' '{print $2}' `
    temp_pacemaker_ver=`rpm -qa pacemaker | head -1 | awk -F'-' '{print $2}' `
    cat > ${TEMP_FILE} << EOF

------  checklist : corosync process  ----------
Corosync  Version : ${temp_corosync_ver}
Pacemaker Version : ${temp_pacemaker_ver}
EOF
    temp_run=` ps axfh | grep -w "corosync$" | wc -l `
    if [ "${temp_run}" -ne 0 ]
    then
        echo "Corosync  Process : online"                            >> ${TEMP_FILE}
        ps axfh | grep -e "corosync" -e "pacemaker" | grep -v "grep" >> ${TEMP_FILE}

        if [[ 1.4.* == ${temp_corosync_ver} ]]
        then
            temp_corosync_pid=` ps axfh | grep -e corosync | grep -v Z | grep -v grep | awk '{print $1}' 2>/dev/null `
            temp_num=0
            for proc in ${pacemaker_process_array[@]}
            do
                temp_pacemaker_pid=` pidof ${proc} `
                check_ppid=` ps -p ${pid:-${temp_pacemaker_pid}} -o ppid= | awk -F' ' '{print $1}' `
                if [ "${check_ppid}" != "${temp_corosync_pid}" ]
                then
                    if [ "${temp_num}" -eq 0 ]
                    then
                        cat ${TEMP_FILE} >> ${CHECK_FILE}
                        echo "------  RESULT(corosync fork): [ wrong ]  ------" >> ${CHECK_FILE}
                        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" ""
                    fi
                    echo -e "${proc}(${temp_pacemaker_pid})\t is not a sub of corosync(${temp_corosync_pid})" >> ${CHECK_FILE}
                    temp_num=1
                fi
            done
            if [ "${temp_num}" -eq 0 ]
            then
                cat ${TEMP_FILE} >> ${CHECK_FILE}
                echo "------  RESULT(corosync fork): [ correct ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
            fi
        elif [[ 2.4.* == ${temp_corosync_ver} ]]
        then
            temp_pacemakerd_pid=` ps axfh | grep -e pacemakerd | grep -v Z | grep -v grep | awk '{print $1}' 2>/dev/null `
            temp_num=0
            for proc in ${pacemaker_process_array[@]}
            do
                temp_pacemaker_pid=` pidof ${proc} `
                check_ppid=` ps -p ${pid:-${temp_pacemaker_pid}} -o ppid= | awk -F' ' '{print $1}' `
                if [ "${check_ppid}" != "${temp_pacemakerd_pid}" ]
                then
                    if [ "${temp_num}" -eq 0 ]
                    then
                        cat ${TEMP_FILE} >> ${CHECK_FILE}
                        echo "------  RESULT(corosync fork): [ wrong ]  ------" >> ${CHECK_FILE}
                        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" ""
                    fi
                    echo -e "${proc}(${temp_pacemaker_pid})\t is not a sub of pacemakerd(${temp_pacemakerd_pid})" >> ${CHECK_FILE}
                    temp_num=1
                fi
            done
            
            if [ "${temp_num}" -eq 0 ]
            then
                cat ${TEMP_FILE} >> ${CHECK_FILE}
                echo "------  RESULT(corosync fork): [ correct ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
            fi
        fi
        cat > ${TEMP_FILE} << EOF

------  checklist : crm_mon  -------------------
EOF
        echo "${check_install} Service :" >> ${TEMP_FILE}
        if [ ${temp_rsync} -eq 2 ]
        then
            crm_mon -A1 2>/dev/null       >> ${TEMP_FILE} # crm_mon -Afr1
        else
            crm_mon -1  2>/dev/null       >> ${TEMP_FILE}
        fi
    else
        echo "Corosync Process : offline" >> ${TEMP_FILE}
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # HA License
    check_mod="[ correct ]"
    if [ -f "${IMOS_CONFIG_DIR}/license/license.dat" ]
    then
        check_license="exist"
    else
        check_license="non-exist"
    fi
    if   [[ ${check_ver} == *B33[0-1]* ]]  # before B3323
    then
        haserver_log="/var/log/logimoshcha00.log"
        hasvrun_log="/var/log/imoslog/vm/vminstall.log"
    elif [[ ${check_ver} == *B332[3-8]* ]] # B3323-B3328
    then
        haserver_log="/var/log/imoslog/haserver00.log"
        hasvrun_log="/var/log/imoslog/hasvrun.log"
    elif [[ ${check_ver} == *B3329* ]]     # B3329
    then
        haserver_log="/var/log/cluster/haserver00.log"
        hasvrun_log="/var/log/cluster/hasvrun.log"
    elif [[ ${check_ver} == *B3335* ]]     # B3335
    then
        haserver_log="/var/log/cluster/haserver00.log"
        hasvrun_log="/var/log/cluster/imos.log"
    else # if not ,default as newest path
        haserver_log="/var/log/cluster/haserver00.log"
        hasvrun_log="/var/log/cluster/imos.log"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : ha license  ----------------
${check_install} License File : ${check_license}
EOF
    if [ ` grep "license " ${haserver_log} 2>/dev/null | tail -1 | grep -c -v "success" ` -ne 1 ]
    then
        check_mod="[ wrong ]"
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    echo "------  RESULT(ha license) : ${check_mod}  ------" >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_mod}" ""


    my_wait # DB Sync
    if [ ${temp_rsync} -eq 2 ]
    then
        check_rsync="rsync"
    elif [ ${temp_rsync} -eq 3 ]
    then
        check_rsync="drbd"
    else
        check_rsync="unknown"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : data sync mode  ------------
DB Sync Method : ${check_rsync}
Status of Sync :
EOF
    # if use drbd
    [ ${temp_rsync} -eq 3 ] && service drbd status >> ${TEMP_FILE} && echo >> ${TEMP_FILE}
    ps axfh | grep postgres | grep -v "postgres: postgres" | grep -v grep  >> ${TEMP_FILE}
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Switch Record
    check_item_num=5
    cat > ${TEMP_FILE} << EOF

------  checklist : switch record  -------------
${hasvrun_log} :
` grep -i "all service" ${hasvrun_log} `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # log daily archive
    cat > ${TEMP_FILE} << EOF

------  checklist : log daily archive ----------
(last ${check_item_num} days) :
` find /var/log/cluster/corosync.log*.gz -mtime -${check_item_num} -type f 2>/dev/null | xargs -I {} ls -lh {} `

` ls -lht /var/log/messages* | head -${check_item_num} `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
    echo "================================================" >> ${CHECK_FILE}
}

VM_Status()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : VM Service
EOF


    my_wait 0 # VM Version
    check_install="not installed"
    check_ver="not installed"
    check_patch="not installed"
    cat > ${TEMP_FILE} << EOF

------  checklist : vm basic info  -------------
System Ver       : ` getconf LONG_BIT` bit
EOF
    if [ -x "/usr/local/bin/vmcfgtool.sh" ]
    then
        check_install="VM"
        check_ver=` /usr/local/bin/vmcfgtool.sh -v `
        check_install="` echo ${check_ver} | head -1 | sed -r 's/.* ([A-Z]{2,5}[0-9]{4,5}).*/\1/' `"
        [ ""X == "${check_install}"X ] && check_install="VM"
        [ -x "/usr/local/bin/displaypatch.sh" ] && check_patch="` /usr/local/bin/displaypatch.sh `"
        echo "VM Install       : ${check_install}"         >> ${TEMP_FILE}
        echo "${check_ver}"                                >> ${TEMP_FILE}
        echo "${check_install} Patch     : ${check_patch}" >> ${TEMP_FILE}
    else
        echo "VM Install : ${check_install}"                    >> ${TEMP_FILE}
        echo "****** not installed VM, skip ******"             >> ${TEMP_FILE}
        echo "================================================" >> ${TEMP_FILE}
        return
    fi
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # VM Process
    # ` vmserver.sh allstatus 2>/dev/null | awk -F' ' '{if($1 ~ /^S/) printf("%-37s  [ %s ]\n",$1,$3);else print $0}' `
    if [ ! -z "` vmcfgtool.sh -v 2>/dev/null `" ] # if VM, check vm service
    then
        cat > ${TEMP_FILE} << EOF

------  checklist : vm service  ----------------
` vmserver.sh allstatus 2>/dev/null | sed "s,\x1B\[[0-9;]*[a-zA-Z],,g" `
EOF
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


        my_wait # VM License
        check_mod="[ correct ]"
        cat > ${TEMP_FILE} << EOF

------  checklist : vm license  ----------------
License Check Log :
` grep "license" /var/log/imoslog/vmserver00.log 2>/dev/null | tail -10 `
EOF
        if [ ` grep "license " /var/log/imoslog/vmserver00.log 2>/dev/null | tail -1 | grep -c -v "success" ` -ne 1 ]
        then
            check_mod="[ wrong ]"
        fi
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        echo "------  RESULT(vm license) : ${check_mod}  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_mod}" ""


        my_wait # VM Msg Queue
        rm -rf /tmp/GeneralCheck_vm.txt 2>/dev/null
        # qry_dt "vmserver" "display CC gm global" "display CC gm msg stat" "display CC sm global" "display CC sm msg stat"
        cat > ${TEMP_FILE} << EOF

------  checklist : vm massage queue  ----------
----------  bp rmod  ----------
EOF
        dt vmserver > /tmp/GeneralCheck_vm.txt 2>/dev/null << DT
display bp rmod
exit
DT
        cat /tmp/GeneralCheck_vm.txt >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}

        sed -i -e '1,6d' -e '$d' /tmp/GeneralCheck_vm.txt
        sed -i -e '$d'           /tmp/GeneralCheck_vm.txt
        check_num=0
        check_num=` wc -l /tmp/GeneralCheck_vm.txt | awk '{print $1}' `
        if [ ${check_num} -gt 1000 ]
        then
            echo "------  RESULT(bp rmod)=${check_num} : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "bp rmod count:${check_num}"
        else
            echo "------  RESULT(bp rmod)=${check_num} : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi


        my_wait # VM BP Rmod
        cat > ${TEMP_FILE} << EOF

----------  bp msgq  ----------
EOF
        dt vmserver > /tmp/GeneralCheck_vm.txt 2>/dev/null << DT
display bp msgq
exit
DT
        cat /tmp/GeneralCheck_vm.txt >> ${TEMP_FILE}
        temp_title=` sed -n '6p' /tmp/GeneralCheck_vm.txt `
        cat ${TEMP_FILE} >> ${CHECK_FILE}

        sed -i -e '1,6d' -e '$d' /tmp/GeneralCheck_vm.txt
        sed -i -e '$d'           /tmp/GeneralCheck_vm.txt
        check_num=0
        check_num=` cat -n /tmp/GeneralCheck_vm.txt | awk -F' ' '{if(($(NF-1))>=30000){print $0}}' | wc -l `
        if [ ${check_num} -gt 0 ]
        then
            # sed -n '6p' /tmp/GeneralCheck_vm.txt >> ${CHECK_FILE}
            echo "------  RESULT(bp msgq)=${check_num} : [ wrong ]  ------" >> ${CHECK_FILE}
            echo -e "${temp_title}"                                         >> ${CHECK_FILE}
            echo "` cat -n /tmp/GeneralCheck_vm.txt | awk -F' ' '{if(($(NF-1))>=30000){print $0}}' | sed 's/^.\{7\}//g' `" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "See details in check.log"
        else
            echo "------  RESULT(bp msgq)=${check_num} : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi


        my_wait # VM IMG Conn
        cat > ${TEMP_FILE} << EOF

----------  img conn  ----------
EOF
        dt vmserver > /tmp/GeneralCheck_vm.txt 2>/dev/null << DT
display img conn
exit
DT
        cat /tmp/GeneralCheck_vm.txt >> ${TEMP_FILE}
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        check_num=0
        check_num=` cat /tmp/GeneralCheck_vm.txt | awk -F' ' '{if(($1)==100){print $0}}' | wc -l ` >> ${CHECK_FILE}
        if [ ${check_num} -gt 500 ]
        then
            echo "------  RESULT(img conn)=${check_num} : [ wrong ]  ------"   >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]"   "See details in check.log"
        else
            echo "------  RESULT(img conn)=${check_num} : [ correct ]  ------" >> ${CHECK_FILE}
            Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
        fi
        rm -rf /tmp/GeneralCheck_vm.txt 2>/dev/null


        my_wait # PG Daily Backup
        check_item_num=3
        check_days=7
        check_mod="[ correct ]"
        cat > ${TEMP_FILE} << EOF

------  checklist : pg daily backup  -----------
Timing Task for PG Data Backup :
` cat /etc/crontab | grep "dbbk.sh" `

Daily Backup Files (last ${check_days}) :
EOF
        if [ -d "${DB_BACKUP_DIR}" ]
        then
            if [ "` find /var/autobackup/database*tar.gz -mtime -${check_days} -type f | wc -l `" -eq 0 ]
            then
                echo "--[error]-- PG Daily Backup : no recent backup file in ${DB_BACKUP_DIR}" >> ${TEMP_FILE}
                ls -lht /var/autobackup/database*tar.gz | head -${check_days}  >> ${TEMP_FILE}
                check_mod="[ wrong ]"
            else
                if [ "` find /var/autobackup/database*tar.gz -mtime -${check_days} -type f | wc -l `" -lt ${check_days} ]
                then
                    check_mod="[ wrong ]"
                fi
                find /var/autobackup/database*tar.gz -mtime -${check_days} -type f | xargs -I {} ls -lh {} >> ${TEMP_FILE}
                echo -e "\nCheck Backup File :" >> ${TEMP_FILE}
                for temp_file in ` find /var/autobackup/database*tar.gz -mtime -${check_item_num} -type f `
                do
                    my_wait
                    echo ${temp_file} >> ${TEMP_FILE}
                    rm -rf /tmp/GeneralCheck_pg.txt 2>/dev/null
                    zcat ${temp_file} | grep -n --binary-files=text -e "Type: DATABASE" -e "database dump complete" -e "MPPV" >/tmp/GeneralCheck_zcat.txt 2>/dev/null & { sleep 20 ; echo -en "   wait...\b\b\b\b\b\b\b\b\b\b" & }
                    wait # waiting for command zcat
                    if [ ! -s /tmp/GeneralCheck_zcat.txt ]
                    then
                        check_mod="[ wrong ]"
                        break
                    fi
                    cat /tmp/GeneralCheck_zcat.txt | grep "Type: DATABASE"         | awk -F "[:;]"  '{printf("LineNo %-10s -- DataBase:%s\n",$1,$3)}'  >> /tmp/GeneralCheck_pg.txt
                    cat /tmp/GeneralCheck_zcat.txt | grep "MPPV" | grep "B33"      | awk -F "[:\t]" '{printf("LineNo %-10s -- Version : %s\n",$1,$5)}' >> /tmp/GeneralCheck_pg.txt
                    cat /tmp/GeneralCheck_zcat.txt | grep "database dump complete" | awk -F ':'     '{printf("LineNo %-10s %s\n",$1,$2)}'              >> /tmp/GeneralCheck_pg.txt
                    sort -n -k 2 -t ' ' /tmp/GeneralCheck_pg.txt >> ${TEMP_FILE}
                    echo >> ${TEMP_FILE}
                    temp_size=` du -shm ${temp_file} | awk -F' ' '{print $1}' `
                    if [ "$temp_size" -gt 600 ] # 10G
                    then
                        echo "PGdata backup file bigger than 600M, skip check" >> ${TEMP_FILE}
                        break
                    fi
                done
                rm -rf /tmp/GeneralCheck_pg.txt   2>/dev/null
                rm -rf /tmp/GeneralCheck_zcat.txt 2>/dev/null
            fi
        else
            echo "--[error]-- PG Daily Backup : no PGdata backup folder ${DB_BACKUP_DIR}" >> ${TEMP_FILE}
            check_mod="[ wrong ]"
        fi
        cat ${TEMP_FILE} >> ${CHECK_FILE}
        echo "------  RESULT(pg daily backup) : ${check_mod}  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "${check_mod}" ""


        my_wait # IPCNum onvif
        pagsyscfg=/usr/local/svconfig/vmconf/pagsyscfg.ini
        if [ -s "${pagsyscfg}" ] # exist and not empty
        then
            temp_ipc_num=` cat ${pagsyscfg} | grep "MaxIPCNum" | awk -F'=' '{print $2}' `
            check_ipc_num=$((${temp_ipc_num}-${IPCNum_onvif_pg}))
            cat > ${TEMP_FILE} << EOF

------  checklist : IPCNum onvif  --------------
onvif IPCNum config:
IPCNum_max_config( A ) = ${temp_ipc_num}
IPCNum_in_reality( B ) = ${IPCNum_onvif_pg}
difference value (A-B) = ${check_ipc_num}
EOF
            cat ${TEMP_FILE} >> ${CHECK_FILE}
            if [ ${check_ipc_num} -lt 0 ]
            then
                echo "------  RESULT(IPCNum onvif) : [ wrong ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "A < B , need increase MaxIPCNum in pagsyscfg.ini"
            elif [ ${check_ipc_num} -gt 1000 ]
            then
                echo "------  RESULT(IPCNum onvif) : [ wrong ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "( A-B ) > 1000 , maybe need decrease MaxIPCNum in pagsyscfg.ini"
            else
                echo "------  RESULT(IPCNum onvif) : [ correct ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
            fi
        fi


        my_wait # IPCNum guobiao
        paggbsyscfg=/usr/local/svconfig/vmconf/paggbsyscfg.ini
        if [ -s "${paggbsyscfg}" ] # exist and not empty
        then
            temp_ipc_num=` cat ${paggbsyscfg} | grep "MaxIPCNum" | awk -F'=' '{print $2}' `
            check_ipc_num=$((${temp_ipc_num}-${IPCNum_guobiao_pg}))
            cat > ${TEMP_FILE} << EOF

------  checklist : IPCNum GB  -----------------
GB IPCNum config:
IPCNum_max_config( A ) = ${temp_ipc_num}
IPCNum_in_reality( B ) = ${IPCNum_guobiao_pg}
difference value (A-B) = ${check_ipc_num}
EOF
            cat ${TEMP_FILE} >> ${CHECK_FILE}
            if [ ${check_ipc_num} -lt 0 ]
            then
                echo "------  RESULT(IPCNum GB) : [ wrong ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "A < B , need increase MaxIPCNum in paggbsyscfg.ini"
            elif [ ${check_ipc_num} -gt 10000 ]
            then
                echo "------  RESULT(IPCNum GB) : [ wrong ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ wrong ]" "( A-B ) > 10000 , maybe need decrease MaxIPCNum in pagsyscfg.ini"
            else
                echo "------  RESULT(IPCNum GB) : [ correct ]  ------" >> ${CHECK_FILE}
                Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "[ correct ]" ""
            fi
        fi
    else
        cat >> ${CHECK_FILE} << EOF

------  checklist : vm service  ----------------
****** VM service not running, skip ******
EOF
    fi
    echo "================================================" >> ${CHECK_FILE}
}

System_Log()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : System Log
EOF


    my_wait 0 # Boot Record
    check_item_num=20
    cat > ${TEMP_FILE} << EOF

------  checklist : boot record  ---------------
(last ${check_item_num}) :
` [ -f "/var/log/wtmp" ] && last -F | grep "reboot\|shutdown" | head -${check_item_num} `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""


    my_wait # Messages Log
    # check lastest 2 message log
    temp_message=(` ls -t /var/log/messages* | head -2 `)
    # echo ${temp_message[@]}
    cat > ${TEMP_FILE} << EOF

------  checklist : message log  ---------------
Messages : Restart Record
` grep -i -e "BIOS-p" -e "system start type" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : I/O error, Medium Error
` grep -i -e "I/O error" -e "Medium Error" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : Out of Memory
` grep -i -e "Out of memory" -e "oom_kill" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : Call Trace
` grep -i -C 7 -e "Call *Trace" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : Read-Only
` grep -i -e "Read-Only" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : Link is Down
` grep -i -e "Link is Down" ${temp_message[@]} | tail -20 `

--------------------------------------------------
Messages : Neighbour table overflow
` grep -i -e "Neighbour table overflow" ${temp_message[@]} | tail -20 `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ ` wc -l ${TEMP_FILE} | awk -F ' ' '{print $1}' ` -gt 100 ]
    then
        Write_to_CHECK_CSV "------  checklist : message log  ---------------" "" "See details in check.log"
    else
        Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" "See details in check.log"
    fi


    my_wait # Mcelog
    # check /var/log/mcelog
    cat >> ${CHECK_FILE} << EOF

------  checklist : mcelog  --------------------
` tail -30 /var/log/mcelog 2>/dev/null `
EOF
    Write_to_CHECK_CSV "------  checklist : mcelog  --------------------" "" "See details in check.log"


    my_wait # Sar Log
    # sar -f sa/sa13  --legacy -s 15:00:00 -e 18:00:00 -p | more
    cat >> ${CHECK_FILE} << EOF

------  checklist : sar log  -------------------
Report CPU utilization :
` sar -u ALL -P ALL 2>/dev/null `

--------------------
Report task creation and system switching activity :
` sar -w 2>/dev/null `

--------------------
Report swapping statistics :
` sar -W 2>/dev/null `

--------------------
Report paging statistics :
` sar -B 2>/dev/null `

--------------------
Report I/O and transfer rate statistics :
` sar -b 2>/dev/null `

--------------------
Report memory statistics :
` sar -R 2>/dev/null `

--------------------
Report memory utilization statistics :
` sar -r 2>/dev/null `

--------------------
Report swap space utilization statistics :
` sar -S 2>/dev/null `

--------------------
Report status of inode, file and other kernel tables :
` sar -v 2>/dev/null `

--------------------
Report queue length and load averages :
` sar -q 2>/dev/null `

--------------------
Report TTY device activity :
` sar -y 2>/dev/null `

--------------------
Report activity for each block device :
` sar -d -p 2>/dev/null `

--------------------
Report network statistics :
` sar -n ALL 2>/dev/null `
EOF
    Write_to_CHECK_CSV "------  checklist : sar log  -------------------" "" "See details in check.log"


    my_wait # Sar Log Keyword
    # sar日志plist-sz数量超过5000，且持续5次记录
    temp_proc=5000
    check_proc=`sar -q 2>/dev/null | sed -n '/^[0-9]/p' | awk -F' ' '{if($4>'${temp_proc}') print $0}' `
    check_num=` sar -q 2>/dev/null | sed -n '/^[0-9]/p' | awk -F' ' '{if($4>'${temp_proc}') print $1,$2," --- proc_count: "$4}' | wc -l `
    # swap space use > 500 MB
    temp_sar_swap_first=`sar -S 2>/dev/null | grep -e "[0-9][0-9]:" | grep -v "kbswpfree" | head -1 | awk '{print $4}' `
    temp_sar_swap_last=` sar -S 2>/dev/null | grep -e "[0-9][0-9]:" | grep -v "kbswpfree" | tail -1 | awk '{print $4}' `
    if [ -z "${temp_sar_swap_first}" -a -z "${temp_sar_swap_last}" ]
    then
        diff=0
    else
        let diff=(${temp_sar_swap_last} - ${temp_sar_swap_first})
    fi
    
    cat > ${TEMP_FILE} << EOF

------  checklist : sar log checkpoint  --------
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    if [ "${check_num}" -gt 4 ]
    then
        echo "------  RESULT(proc count)>${temp_proc} : [ wrong ]  ------" >> ${CHECK_FILE}
        echo -e "${check_proc} \n"                                         >> ${CHECK_FILE}
        Write_to_CHECK_CSV "proc count" "[ wrong ]"   "${check_proc}"
    else
        echo "------  RESULT(proc count) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "proc count" "[ correct ]" ""
    fi
    if [ ${diff} -ge 512000 ]   # 500MB = 500*1024KB
    then
        echo "------  RESULT(swap increase)>500MB : [ wrong ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "swap increase" "[ wrong ]"   "TodayDate swap increase is ${diff}"
    else
        echo "------  RESULT(swap increase) : [ correct ]  ------" >> ${CHECK_FILE}
        Write_to_CHECK_CSV "swap increase" "[ correct ]" ""
    fi

    echo "================================================" >> ${CHECK_FILE}
}

Tool_Deployment()
{
    cat >> ${CHECK_FILE} << EOF

================================================
Check Time : ` date `
Check Item : System Tool
EOF
    my_wait 0 # Watchdog Patch Check
    Watchdog_Patch
    my_wait # Coredump Check
    Coredump_Check
    my_wait # Tcpdump Check
    Tcpdump_Check

    cat >> ${CHECK_FILE} << EOF
================================================
EOF
}

Watchdog_Patch()
{
    if [ -d "/usr/local/hstor" ]
    then
        check_watchdog="installed"
    else
        check_watchdog="not install"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : watchdog  ------------------
Watchdog Patch : ${check_watchdog}
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
}

Coredump_Check()
{
    if [ "` ulimit -c `" == "0" ]
    then
        check_coredump="off"
    else
        check_coredump="on"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : coredump  ------------------
Coredump Status : ${check_coredump}
Core File Path  : ` cat /proc/sys/kernel/core_pattern `
Coredump config :

` grep -e "soft" -e "hard" /etc/security/limits.conf | sed 's/"//g' `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
}

Tcpdump_Check()
{
    if [ -f "/usr/local/bin/vm_msg.sh" ]
    then
        check_coredump_install="installed"
    else
        check_coredump_install="not install"
    fi
    cat > ${TEMP_FILE} << EOF

------  checklist : tcpdump  -------------------
Tcpdump Install : ${check_coredump_install}
` sh /usr/local/bin/vm_msgserver.sh status 2>/dev/null || du -sh /var/vm_msg 2>/dev/null `
EOF
    cat ${TEMP_FILE} >> ${CHECK_FILE}
    Write_to_CHECK_CSV "`cat ${TEMP_FILE}`" "" ""
}

Install_Sshpass()
{
    cmd=` sshpass -V 2>/dev/null `
    if [ $? -ne 0 ]
    then
        OLD_PATH=` pwd `
        # install sshpass-1.06
        cd ${SHELL_DIR}/sshpass*
        ./configure  > /dev/null 2>&1
        make         > /dev/null 2>&1
        make install > /dev/null 2>&1
        [ $? -ne 0 ] && echo " --error-- : install sshpass failed" && My_Exit 1
        cd ${OLD_PATH}
    fi
}

Ssh_to_Execute_Check()
{
    # $1 : check ip
    # $2 : password
    # $3 : ssh port

    ssh_trust=1 # 0 : ssh trust ok
    # check whether install sshpass
    Install_Sshpass

    temp_psw=$2
    temp_port=$3
    # first step, test whether there has ssh trust
    ssh $1 -p ${temp_port} -o PreferredAuthentications=publickey -o StrictHostKeyChecking=no "uname -a" > /dev/null 2>&1
    if [ $? -ne 0 ] # no ssh trust
    then
        # second step, check root password
        if [ ""X != "${temp_psw}"X ]
        then # password exist
            sshpass -p "${temp_psw}" ssh -p ${temp_port} -o StrictHostKeyChecking=no root@$1 "uname -a" > /dev/null 2>&1
            if [ $? -ne 0 ]
            then # password is not correct
                if [ "${TIMING_JOB}" -ne 1 ]
                then # if it called by crontab, then retuen 1
                    echo " --error-- : $1 can not connect, skip"
                    return 1
                else # third step, input password manually
                    while true
                    do
                        stty -echo
                        echo "please input $1 root password :"
                        read temp_psw
                        stty echo
                        sshpass -p "${temp_psw}" ssh -p ${temp_port} -o StrictHostKeyChecking=no root@$1 "uname -a" > /dev/null
                        [ $? -eq 0 ] && break
                    done
                fi
            fi
        else # password not exist
            if [ "${TIMING_JOB}" -ne 1 ]
            then # if it called by crontab, then retuen 1
                echo " --error-- : $1 can not connect"
                return 1
            else # third step, input password manually
                while true
                do
                    stty -echo
                    echo "Can not connect $1 ,please input root password :"
                    read temp_psw
                    stty echo
                    sshpass -p "${temp_psw}" ssh -p ${temp_port} -o StrictHostKeyChecking=no root@$1 "uname -a" > /dev/null
                    [ $? -eq 0 ] && break
                done
            fi
        fi
    else
        ssh_trust=0
    fi

    if [ "${ssh_trust}" -eq 0 ]
    then
        echo "Transfering script ...  "
        scp -P ${temp_port} -pq ${SHELL_PATH} root@$1:/home/GeneralCheck_copy.sh 2>/dev/null
        # font color red
        [ $? -ne 0 ] && echo -e "\033[31m\033[1m[error]\033[0m ip : $1 transfer failed  ----------" && return 1

        echo "Running script ...  "
        if [ ${CHECK_EXPIRED} -eq 0 ]
        then
            ssh $1 -p ${temp_port} "sh /home/GeneralCheck_copy.sh -p $1 -n ${check_item}    " 2>/dev/null
        else
            ssh $1 -p ${temp_port} "sh /home/GeneralCheck_copy.sh -p $1 -n ${check_item} -e " 2>/dev/null
        fi
        # font color red
        [ $? -ne 0 ] && echo -e "\033[31m\033[1m[error]\033[0m ip : $1 execute failed  ----------" && return 1

        echo "Collecting data ...  "
        scp -P ${temp_port} -pq root@$1:/home/$1_check.csv ${LOG_DIR}/ 2>/dev/null
        scp -P ${temp_port} -pq root@$1:/home/$1_check.log ${LOG_DIR}/ 2>/dev/null
        scp -P ${temp_port} -pq root@$1:/home/$1_svconfig.tgz ${LOG_DIR}/ 2>/dev/null
        [ $? -eq 0 ] && echo "----- ip : $1 collected -----"
    else
        echo "Transfering script ...  "
        sshpass -p "${temp_psw}" scp -P ${temp_port} -pq ${SHELL_PATH} root@$1:/home/GeneralCheck_copy.sh 2>/dev/null
        # font color red
        [ $? -ne 0 ] && echo -e "\033[31m\033[1m[error]\033[0m ip : $1 transfer failed  ----------" && return 1

        echo "Running script ...  "
        if [ ${CHECK_EXPIRED} -eq 0 ]
        then
            sshpass -p "${temp_psw}" ssh $1 -p ${temp_port} "sh /home/GeneralCheck_copy.sh -p $1 -n ${check_item}    " 2>/dev/null
        else
            sshpass -p "${temp_psw}" ssh $1 -p ${temp_port} "sh /home/GeneralCheck_copy.sh -p $1 -n ${check_item} -e " 2>/dev/null
        fi
        # font color red
        [ $? -ne 0 ] && echo -e "\033[31m\033[1m[error]\033[0m ip : $1 execute failed  ----------" && return 1

        echo "Collecting data ...  "
        sshpass -p "${temp_psw}" scp -P ${temp_port} -pq root@$1:/home/$1_check.csv ${LOG_DIR}/ 2>/dev/null
        sshpass -p "${temp_psw}" scp -P ${temp_port} -pq root@$1:/home/$1_check.log ${LOG_DIR}/ 2>/dev/null
        sshpass -p "${temp_psw}" scp -P ${temp_port} -pq root@$1:/home/$1_svconfig.tgz ${LOG_DIR}/ 2>/dev/null
        [ $? -eq 0 ] && echo "----- ip : $1 collected -----"
        unset temp_psw # release password
    fi
    return 0
}

Handle_Check_FIle()
{
    # tar all servers check log file
    now_time=` date +"%Y%m%d_%H%M" `
    tar_name="${LOG_DIR}/${now_time}_check_log.tar.gz"
    OLD_PATH=` pwd `
    echo
    echo -n "tar log to ${tar_name} "
    cd ${LOG_DIR}
    tar zcf ${tar_name} ${check_log_array[@]} ${check_csv_array[@]} *svconfig.tgz
    if [ $? -eq 0 ]
    then
        # font color green
        printf "\033[32m\033[1m[ done ]\033[0m\n"
        echo "check_log.tar.gz file list:"
        tar tvf ${tar_name}
        [ "${TIMING_JOB}" -ne 1 ] && /bin/rm -rf ${check_log_array[@]} ${check_csv_array[@]} 2>/dev/null
    else
        # font color red
        printf "\r\t\t\t\t\t\033[31m\033[1m[ failed ]\033[0m\n"
    fi
}

Check_Main()
{
    echo "========================================"
    date
    echo "--------  ip : ${IP_INFO} checking "
    CHECK_FILE="${LOG_DIR}/${IP_INFO}_${CHECK_FILE}"
    CHECK_CSV="${LOG_DIR}/${IP_INFO}_${CHECK_CSV}"
    echo > ${CHECK_FILE}
    echo > ${CHECK_CSV}
    echo "------------ check result as follow ------------" >> ${CHECK_FILE}
    echo "    ` date `"                                     >> ${CHECK_FILE}
    echo "    IP : ${IP_INFO}"                              >> ${CHECK_FILE}
    echo "    ${MY_VERSION}"                                >> ${CHECK_FILE}
    index=1
    for item in ${check_item_array[@]}
    do
        if [ "${check_item}" == "all" ] || [[ ${check_item} == *${index}* ]]
        then
            # echo -n -e "check ${item/_/ }\t\t"
            printf 'check %-20s   ' "${item/_/ }"
            ${item}
            # font color green
            printf "\r\t\t\t\t\033[32m\033[1m[ done ]\033[0m\n"
        fi
        let "index+=1"
    done
    
    tar czvf ${LOG_DIR}/${IP_INFO}_svconfig.tgz  /usr/local/imosconfig > /dev/null 2>&1

    echo "check result( ${CHECK_FILE} ) created."
    echo "--------  ip : ${IP_INFO} finished "
    echo "========================================"
    return 0
}

Check_Unique_Instance()
{
    if [ -f "${PID_FILE}" ]
    then
        temp_pid=` cat ${PID_FILE} `
        if [ -n "` ps axfh | grep "${temp_pid}" | grep "GeneralCheck" `" ]
        then
            # pid exist , kill process
            kill -9 ${temp_pid} > /dev/null 2>&1
        fi
    fi
    # write new pid
    echo $$ > ${PID_FILE} # write current script process
}

My_Exit()
{
    # $1 : exit code
    /bin/rm -rf ${PID_FILE} 2>/dev/null
    cd ${OLD_PATH}
    exit $1
}

Usage()
{
    cat << USAGE
Usage :
  sh GeneralCheck.sh -p local_ip    ---- check local service
  sh GeneralCheck.sh -f ip_file     ---- check 2 or more services
  sh GeneralCheck.sh -n item_num    ---- check items with option -n
      (-n): 1  System_Info
            2  Process_Status
            3  Disk_Space
            4  Network_Status
            5  PG_Status
            6  HA_Status
            7  VM_Status
            8  System_Log
            9  Tool_Deployment
  sh GeneralCheck.sh -v             ---- see version
  sh GeneralCheck.sh -e             ---- ignore version expired
  sh GeneralCheck.sh -h             ---- show this usage
USAGE
    exit
}

#################### MAIN ####################
check_single=0
check_item="all"
ip_array=()
port_array=()
psw_array=()
if [ $# -eq 0 ] # no para, show usage
then
    Usage
else
    para_in=` getopt f:p:n:tveh "$@" `
    [ $? -ne 0 ] && exit 1
    eval set -- "${para_in}"
    while true
    do
        case "$1" in
        -f) # ip address file
            check_single=1 # 1 : check more than 1 servers & tar XXX_check.log
            if [ -f "$2" ]
            then
                n=0
                sed -i /^$/d "$2"
                while read line
                do
                    ip_array[n]=` echo "${line}" | awk -F' ' '{print $1}' | awk -F':' '{print $1}' `
                    port_array[n]=` echo "${line}" | awk -F' ' '{print $1}' | awk -F':' '{print $2}' 2>/dev/null `
                    [ -z "${port_array[n]}" ] && port_array[n]=22
                    psw_array[n]=` echo "${line}" | awk -F' ' '{print $2}' 2>/dev/null `
                    [ "" == "${psw_array[n]}" ] && psw_array[n]="null"
                    let "n+=1"
                done < "$2"
                # echo "${ip_array[@]}"
                # echo "${port_array[@]}"
                # echo "${psw_array[@]}"
            else
                echo -e "\n--[error]-- $2 is not a file \n"
                exit 1
            fi
            shift
            ;;
        -p) # single ip address
            check_single=0 # 0 : check only 1 server & no tar XXX_check.log
            temp_ip=${2%%:*} # get what ahead of ':'
            temp_port=` echo "${t}" | awk -F':' '{print $2}' ` # get what behind of ':'
            [ -z "${temp_port}" ] && temp_port=22
            ip_array=("${temp_ip}")
            port_array=("${temp_port}")
            psw_array=("null")
            # echo "${temp_ip}--${temp_port}"
            shift
            ;;
        -n) # choose check items
            check_item=${2}
            shift
            ;;
        -t) # timing-job mode
            TIMING_JOB=0 # 0 : timing-job mode on
            LOG_DIR="/var/log/general_check"
            [ ! -d "${LOG_DIR}" ] && mkdir -p ${LOG_DIR}
            ;;
        -v) # see version
            echo " ${MY_VERSION}"
            exit 0
            ;;
        -e) # ignore expired
            CHECK_EXPIRED=1 # 1 : do not check
            ;;
        -h) # see usage
            Usage
            ;;
        --) break
            ;;
        *) 
            exit 1
            ;;
        esac
        shift
    done

    if [ ${CHECK_EXPIRED} -eq 0 ]
    then
        # check whether expired
        ReleaseDate=` echo "${MY_VERSION}" | awk -F' ' '{print $6}' `
        TodayDate=` date +%s `
        DiffDays=$(( ( ${TodayDate} - $(date -d "${ReleaseDate}" +%s) )/86400  ))
        # echo "DiffDays=${DiffDays}"
        if [ ${DiffDays} -gt 180 ]
        then
            # font color red
            echo -e "\033[31m\033[1mThis script version(${MY_VERSION}) is too old, please take a newest one \033[0m"
            exit 0
        fi
    fi

    Check_Unique_Instance
    echo "${MY_VERSION}"
    # font color green
    echo -e "\033[32m\033[1mThe number of ip addr : ${#ip_array[@]}\033[0m"
    for ((ip_index=0 ; ip_index < ${#ip_array[@]} ; ip_index++))
    do
        ifconfig | grep -w "${ip_array[ip_index]}" > /dev/null
        if [ $? -eq 0 ]
        then
            # echo "ip : ${ip_array[ip_index]} is a local ip"
            # Get_IP ${ip_array[ip_index]}
            IP_INFO=${ip_array[ip_index]}
            Check_Main
            ret_code=$?
        else
            echo -e "\nip : ${ip_array[ip_index]} is a remote ip"
            ping -W 1 -c 1 ${ip_array[ip_index]} > /dev/null
            if [ $? -eq 0 ]
            then
                echo "ip : ${ip_array[ip_index]} available"
            else
                # font color red
                echo -e "\033[31m\033[1mip : ${ip_array[ip_index]} NOT available\033[0m"
                continue
            fi
            Ssh_to_Execute_Check ${ip_array[ip_index]} ${psw_array[ip_index]} ${port_array[ip_index]}
            ret_code=$?
        fi
        echo
        if [ "${check_single}" -eq 1 ] && [ "${ret_code}" -eq 0 ]
        then
            check_log_array[${#check_log_array[@]}]="${ip_array[ip_index]}_check.log"
            check_csv_array[${#check_csv_array[@]}]="${ip_array[ip_index]}_check.csv"
        fi
    done
    # tar check log
    [ ${check_single} -eq 1 ] && Handle_Check_FIle
    My_Exit 0
fi

# netstat -a -o

# beep
# Mar 29 15:14:39 localhost alarmd: [ALARM_INFO] Change beep_value to ON, cur_beep_flag: 1
# grep -C 3 "beep_value" /var/log/message

# 硬件报错
# cat /var/log/mcelog

# ps axfh | grep -A 5 "GeneralCheck"
# tar zcf GeneralCheck_20221019.tar.gz ./GeneralCheck


